package com.ekexiu.portal.article.file;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.Upload;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.web.fileupload.Item;
import org.jfw.util.web.fileupload.UploadItemIterator;

@Path("/article/files")
public class ArticleFileService {
	private static final AtomicInteger FN_IDX = new AtomicInteger(1);
	private static final AtomicInteger CLEAN_FLAG = new AtomicInteger(0);

	private File imgPath;

	@Autowrie
	private ArticleFileDao articleFileDao;

	public File getImgPath() {
		return imgPath;
	}

	public void setImgPath(File imgPath) {
		this.imgPath = imgPath;
	}

	public ArticleFileDao getArticleFileDao() {
		return articleFileDao;
	}

	public void setArticleFileDao(ArticleFileDao articleFileDao) {
		this.articleFileDao = articleFileDao;
	}

	public void update(Connection con, String id, String[] files) throws SQLException {

		articleFileDao.delete(con, id);
		if (files != null && files.length > 0) {
			ArticleFile[] fs = new ArticleFile[files.length];
			for (int i = 0; i < files.length; ++i) {
				ArticleFile file = new ArticleFile();
				fs[i] = file;
				String fp = files[i];
				int idx = 0;

				if (fp == null) {
					throw new IllegalArgumentException("paser article file error: null");
				}
				idx = fp.indexOf(",");
				if (idx == (-1)) {
					throw new IllegalArgumentException("paser article file error: " + files[i]);
				}
				file.setUrl(fp.substring(0, idx));
				fp = fp.substring(++idx);
				idx = fp.indexOf(",");
				if (idx == (-1)) {
					throw new IllegalArgumentException("paser article file error: " + files[i]);
				}
				file.setSeq(Integer.parseInt(fp.substring(0, idx)));
				fp = fp.substring(++idx);
				idx = fp.indexOf(",");
				if (idx == (-1)) {
					throw new IllegalArgumentException("paser article file error: " + files[i]);
				}
				file.setSize(Long.parseLong(fp.substring(0, idx)));
				fp = fp.substring(++idx);
				file.setName(fp);
				file.setId(id);
			}
			articleFileDao.insert(con, fs);
		}
	}

	@Get
	@Path("/byArticleId")
	public List<ArticleFile> query(@JdbcConn Connection con, String id) throws SQLException {
		return articleFileDao.query(con, id);
	}

	private void clean(Connection con, File path, String prefix) throws SQLException {
		for (File file : path.listFiles()) {
			String url = prefix + "/" + file.getName();
			if (file.isFile()) {
				if (null == articleFileDao.queryByurl(con, url)) {
					file.delete();
				}
			} else {
				clean(con, file, url);
			}
		}
	}

	@Get
	@Path("/clean")
	public void clean() throws SQLException {
		JfwAppContext.getScheduledExecutorService().execute(new Runnable() {

			@Override
			public void run() {
				if (CLEAN_FLAG.compareAndSet(0, 1)) {
					Connection con = null;
					try {
						con = JfwAppContext.getDataSource().getConnection();
						clean(con, imgPath, "");
					} catch (Exception e) {
					} finally {
						if (con != null) {
							JdbcUtil.close(con);
							con = null;
						}
						CLEAN_FLAG.set(0);
					}
				}
			}
		});

	}

	@Path("/upload")
	@Post
	public List<UploadFile> upload(@Upload UploadItemIterator it) throws Exception {
		List<UploadFile> ret = new LinkedList<UploadFile>();
		try {
			while (it.hasNext()) {
				Item item = it.next();
				if (!item.isFormField()) {
					String name = normalizeFileName(item.getName());
					int index = name.lastIndexOf('.');
					String ext = index >= 0 ? name.substring(index + 1) : "";
					ext = ext.trim();
					long fileLen = 0;
					int len = 0;
					UploadFile uf = this.buildTargetFile(ext);
					uf.setName(name);
					File file = uf.getFn();
					ret.add(uf);
					byte[] buf = new byte[8092];
					OutputStream of = new FileOutputStream(file);
					try {
						InputStream in = item.getInputStream();
						try {
							while ((len = in.read(buf)) >= 0) {
								if (len > 0) {
									of.write(buf, 0, len);
									fileLen += len;
								}
							}
							uf.setSize(fileLen);
						} finally {
							in.close();
						}
					} finally {
						of.close();
					}
				}
			}
			return ret;
		} catch (Exception e) {
			this.remove(ret);
			throw e;
		} finally {
			if (it != null)
				it.clean();
		}
	}

	private void remove(List<UploadFile> list) {
		for (UploadFile file : list) {
			if (file.fn != null)
				file.fn.delete();
		}
	}

	private UploadFile buildTargetFile(String ext) {
		StringBuilder sb = new StringBuilder();
		sb.append(System.currentTimeMillis() / (1000 * 60 * 60 * 24));
		File path = new File(this.imgPath, sb.toString());
		if (!path.exists()) {
			synchronized (this) {
				if (!path.mkdirs())
					throw new RuntimeException("mkdir error[" + path + "]");
				FN_IDX.set(1);
			}
		}
		File file;
		do {
			String fn = FN_IDX.toString();
			if (ext.isEmpty()) {
				file = new File(path, fn);
			} else {
				file = new File(path, fn + "." + ext);
			}
			FN_IDX.incrementAndGet();
		} while (file.exists());
		sb.append("/").append(file.getName());
		UploadFile uf = new UploadFile();
		uf.setFn(file);
		uf.setUri("/" + sb.toString());
		return uf;
	}

	private String normalizeFileName(String fn) {
		int index = fn.indexOf('\\');
		if (index >= 0) {
			fn = fn.substring(fn.lastIndexOf('\\') + 1);
		}
		index = fn.indexOf('/');
		if (index >= 0) {
			fn = fn.substring(fn.lastIndexOf('/') + 1);
		}
		if (fn.length() == 0)
			throw new RuntimeException("invalid filename in Multipart/data request");
		return fn;
	}

	public static class UploadFile {
		private String name;
		private String uri;
		private long size;
		private transient File fn;

		public File getFn() {
			return fn;
		}

		public void setFn(File fn) {
			this.fn = fn;
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public String getUri() {
			return uri;
		}

		public void setUri(String uri) {
			this.uri = uri;
		}

		public long getSize() {
			return size;
		}

		public void setSize(long size) {
			this.size = size;
		}
	}

}
