package com.ekexiu.portal.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.util.exception.JfwBaseException;

import com.ekexiu.portal.po.Operation;
import com.ekexiu.portal.po.PayHistory;
import com.ekexiu.portal.po.Professor;

@DAO
public abstract class PayHistoryDao {
	@Insert
	public abstract int insert(Connection con, PayHistory payHistory) throws SQLException;
	
	@UpdateWith
	@From(PayHistory.class)
	public abstract int update(Connection con, String payHistoryId, @Set int payStatus) throws SQLException;
	
	public PayHistory query(Connection con,String payHistoryId) throws SQLException, JfwBaseException{
        int index = 1;
        String sql = "SELECT PAY_HISTORY_ID,OPERATION_ID,PAID_MONEY,RETURN_MONEY,REMARK,ISONLINE,PAY_TYPE,"
        		+ " PAY_STATUS,PAY_HISTORY.PROFESSOR_ID,PAY_HISTORY.CREATE_TIME,PROFESSOR.NAME "
        		+ " FROM PAY_HISTORY LEFT JOIN PROFESSOR ON PAY_HISTORY.PROFESSOR_ID = PROFESSOR.ID "
        		+ " WHERE PAY_HISTORY_ID = ?";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,payHistoryId);
            ResultSet rs = ps.executeQuery();
            try{
                if(rs.next()){
                   PayHistory payHistory = new PayHistory();
                    payHistory.setPayHistoryId(rs.getString(1));
                    payHistory.setOperationId(rs.getString(2));
                    payHistory.setPaidMoney(rs.getBigDecimal(3));
                    payHistory.setReturnMoney(rs.getBigDecimal(4));
                    String remark = rs.getString(5);
                    if(rs.wasNull()){
                    	remark = null;
                    }
                    payHistory.setRemark(remark);
                    payHistory.setIsonline(rs.getInt(6));
                    payHistory.setPayType(rs.getInt(7));
                    payHistory.setPayStatus(rs.getInt(8));
                    Professor professor = new Professor();
                    professor.setId(rs.getString(9));
                    payHistory.setCreateTime(rs.getString(10));
                    professor.setName(rs.getString(11));
                    payHistory.setProfessor(professor);
                    return payHistory;
                }else{
                    throw new JfwBaseException(201);
                }
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
    }
	
	public List<PayHistory> queryDetail(Connection con,String operationId) throws SQLException{
        int index = 1;
        String sql = "SELECT PAY_HISTORY_ID,PAY_HISTORY.OPERATION_ID,PAID_MONEY,RETURN_MONEY,REMARK,ISONLINE,"
        		+ " PAY_TYPE,PAY_STATUS,PAY_HISTORY.CREATE_TIME,OPERATION.DEAL_PRICE,OPERATION.CREATE_TIME,"
        		+ " PAY_HISTORY.PROFESSOR_ID,PROFESSOR.NAME "
        		+ " FROM PAY_HISTORY LEFT JOIN OPERATION ON PAY_HISTORY.OPERATION_ID = OPERATION.OPERATION_ID "
        		+ " LEFT JOIN PROFESSOR ON PAY_HISTORY.PROFESSOR_ID = PROFESSOR.ID "
        		+ " WHERE PAY_HISTORY.OPERATION_ID = ?";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
            ps.setString(index++,operationId);
            ResultSet rs = ps.executeQuery();
            try{
                List<PayHistory> payHistories = new ArrayList<PayHistory>();
                while(rs.next()){
                    PayHistory payHistory =  new PayHistory();
                    payHistory.setPayHistoryId(rs.getString(1));
                    Operation operation = new Operation();
                    operation.setOperationId(rs.getString(2));
                    payHistory.setPaidMoney(rs.getBigDecimal(3));
                    payHistory.setReturnMoney(rs.getBigDecimal(4));
                    String remark = rs.getString(5);
                    if(rs.wasNull()){
                    	remark = null;
                    }
                    payHistory.setRemark(remark);
                    payHistory.setIsonline(rs.getInt(6));
                    payHistory.setPayType(rs.getInt(7));
                    payHistory.setPayStatus(rs.getInt(8));
                    payHistory.setCreateTime(rs.getString(9));
                    operation.setDealPrice(rs.getBigDecimal(10));
                    operation.setCreateTime(rs.getString(11));
                    payHistory.setOperation(operation);
                    Professor professor = new Professor();
                    professor.setId(rs.getString(12));
                    professor.setName(rs.getString(13));
                    payHistory.setProfessor(professor);
                    payHistories.add(payHistory);
                }
                return payHistories;
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
    }
	
}
