package com.ekexiu.portal.dao;

import com.ekexiu.portal.po.Demand;
import com.ekexiu.portal.po.Organization;
import com.ekexiu.portal.po.Professor;
import com.ekexiu.portal.pojo.EditProfessor;
import com.ekexiu.portal.pojo.ProfessorInfo;
import com.ekexiu.portal.pojo.SortedPro;
import com.ekexiu.portal.pojo.UserInfo;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.Column;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.LimitColumn;
import org.jfw.apt.orm.annotation.dao.method.Or;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.Select;
import org.jfw.apt.orm.annotation.dao.method.SetSentence;
import org.jfw.apt.orm.annotation.dao.method.Where;
import org.jfw.apt.orm.annotation.dao.method.operator.DeleteWith;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.LimitQuery;
import org.jfw.apt.orm.annotation.dao.method.operator.PageQuery;
import org.jfw.apt.orm.annotation.dao.method.operator.PageSelect;
import org.jfw.apt.orm.annotation.dao.method.operator.QueryList;
import org.jfw.apt.orm.annotation.dao.method.operator.QueryOne;
import org.jfw.apt.orm.annotation.dao.method.operator.QueryVal;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.Update;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.GroupSqlColumn;
import org.jfw.apt.orm.annotation.dao.param.In;
import org.jfw.apt.orm.annotation.dao.param.LessThan;
import org.jfw.apt.orm.annotation.dao.param.Like;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.apt.orm.annotation.dao.param.SqlColumn;
import org.jfw.apt.orm.core.defaultImpl.BigDecimalHandler;
import org.jfw.apt.orm.core.defaultImpl.IntHandler;
import org.jfw.apt.orm.core.defaultImpl.StringHandler;
import org.jfw.util.PageQueryResult;
import org.jfw.util.exception.JfwBaseException;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@DAO
public abstract class ProfessorDao {
	@Insert
	public abstract int insert(Connection con, Professor professor) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	@SetSentence("PAGE_VIEWS = PAGE_VIEWS + 1")
	public abstract int incPageViews(Connection con, String id) throws SQLException;

	@Update
	public abstract int update(Connection con, Professor professor) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract  int updateGrowthScore(Connection con, String id,@Set("= SCORE_VALUE + ?") int scoreValue,@Set("=GROWTH_VALUE + ?") int growthValue ) throws SQLException;


	@UpdateWith
	@From(Professor.class)
	public abstract int updateOrg(Connection con,@Set String orgId,String id)throws SQLException;

	@SelectOne
	@Nullable
	public abstract Professor query(Connection con, long shareId) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updatePhone(Connection con, String id, @Set String phone) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateEmail(Connection con, String id, @Set String email) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateSortFirst(Connection con, String id, @Set Integer sortFirst) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateAuthentication(Connection con, String id, @Set Integer authentication, @Set Integer authType, @Set Integer authStatus)
			throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateAuthType(Connection con, String id, @Set Integer authType) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateAuthentication(Connection con, String id, @Set Integer authentication) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateAuthStatus(Connection con, String id, @Set Integer authStatus) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateAuthStatusExpert(Connection con, String id, @Set Integer authStatusExpert) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateOrgAuth(Connection con, String id, @Set String orgAuth) throws SQLException;
	
	@UpdateWith
	@From(Professor.class)
	public abstract int joinAndPassOrgAuth(Connection con, String id,@Set String orgId, @Set String orgAuth) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateStarLevel(Connection con, String id, @Set BigDecimal starLevel) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateConsultCount(Connection con, String id, @Set Integer consultCount) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateSubject(Connection con, String id, @Set String subject) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateDescp(Connection con, String id, @Set String descp) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateIndustry(Connection con, String id, @Set String industry) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateAddress(Connection con, String id, @Set String address) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateProvince(Connection con, String id, @Set String province) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateStarAvg(Connection con, String id, @Set BigDecimal starAvg) throws SQLException;

	@UpdateWith
	@From(Professor.class)
	public abstract int updateScorePercent(Connection con, String id, @Set String scorePercent) throws SQLException;

	/**
	 * 查用户总数
	 * 
	 * @param con
	 * @return
	 * @throws SQLException
	 */
	@QueryVal
	@Column(handlerClass = IntHandler.class, value = { "COUNT(1)" })
	@From(Professor.class)
	@DefaultValue("0")
	public abstract int queryCount(Connection con) throws SQLException;

	/**
	 * 查询积分值低于scoreValue的用户总数
	 * 
	 * @param con
	 * @param scoreValue
	 *            积分值
	 * @return
	 * @throws SQLException
	 */
	@QueryVal
	@Column(handlerClass = IntHandler.class, value = { "COUNT(1)" })
	@From(Professor.class)
	@DefaultValue("0")
	public abstract int queryCountLessThan(Connection con, @LessThan int scoreValue) throws SQLException;

	@SelectList
	public abstract List<Professor> queryList(Connection con) throws SQLException;

	public List<EditProfessor> queryInvite(Connection con, String id) throws SQLException {
		int index = 1;
		String sql = "SELECT OFFICE,ADDRESS,ORG_ID,O.NAME,TITLE,AUTHENTICATION,P.AUTH_TYPE,P.AUTH_STATUS,P.ID,P.NAME,ORG_AUTH,P.SHARE_ID "
				+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION O ON P.ORG_ID=O.ID "
				+ " INNER JOIN LUSER L ON P.ID=L.ID WHERE L.INVITER_ID = ? ORDER BY P.CREATE_TIME DESC,P.ID";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, id);
			ResultSet rs = ps.executeQuery();
			try {
				List<EditProfessor> professors = new ArrayList<EditProfessor>();
				while (rs.next()) {
					EditProfessor professor = new EditProfessor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String address = rs.getString(2);
					if (rs.wasNull()) {
						address = null;
					}
					professor.setAddress(address);
					Organization organization = new Organization();
					String orgId = rs.getString(3);
					if (rs.wasNull()) {
						orgId = null;
					}
					organization.setId(orgId);
					String orgName = rs.getString(4);
					if (rs.wasNull()) {
						orgName = null;
					}
					organization.setName(orgName);
					professor.setOrganization(organization);
					professor.setTitle(rs.getString(5));
					Integer auth = rs.getInt(6);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(7));
					professor.setAuthStatus(rs.getInt(8));
					professor.setId(rs.getString(9));
					professor.setName(rs.getString(10));
					professor.setOrgAuth(rs.getString(11));
					professor.setShareId(rs.getLong(12));
					professors.add(professor);
				}
				return professors;
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
	}

	/**
	 * 查询用户的认证状态。
	 * 
	 * @param con
	 * @param id
	 * @return 返回认证类型 用户类型 实名认证状态和专家认证状态
	 * @throws SQLException
	 */
	public Professor queryAuth(Connection con, String id) throws SQLException {
		int index = 1;
		String sql = " SELECT AUTHENTICATION,AUTH_TYPE,AUTH_STATUS,AUTH_STATUS_EXPERT,ORG_AUTH FROM PROFESSOR WHERE ID = ? ";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, id);
			ResultSet rs = ps.executeQuery();
			try {
				if (rs.next()) {
					Professor professor = new Professor();
					Integer auth = rs.getInt(1);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(2));
					professor.setAuthStatus(rs.getInt(3));
					professor.setAuthStatusExpert(rs.getInt(4));
					professor.setOrgAuth(rs.getString(5));
					return professor;
				} else {
					return null;
				}
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
	}

	/**
	 * 查询专家基本信息接口
	 * 
	 * @param con
	 * @param id
	 *            专家ID
	 * @return 返回专家基本信息
	 * @throws SQLException
	 */
	public EditProfessor queryBaseInfo(Connection con, String id) throws SQLException {
		int index = 1;
		String sql = "SELECT P.OFFICE,P.DEPARTMENT,P.TITLE,AUTHENTICATION,AUTH_TYPE,P.AUTH_STATUS,"
				+ " P.ID,P.NAME,P.ADDRESS,P.ORG_AUTH,ORGANIZATION.NAME,P.ORG_ID,P.SCORE_PERCENT,P.SHARE_ID,P.PHONE,P.EMAIL,P.PROVINCE"
				+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION ON P.ORG_ID = ORGANIZATION.ID WHERE P.ID = ?";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, id);
			ResultSet rs = ps.executeQuery();
			try {
				if (rs.next()) {
					EditProfessor professor = new EditProfessor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String depart = rs.getString(2);
					if (rs.wasNull()) {
						depart = null;
					}
					professor.setDepartment(depart);
					String title = rs.getString(3);
					if (rs.wasNull()) {
						title = null;
					}
					professor.setTitle(title);
					Integer auth = rs.getInt(4);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(5));
					professor.setAuthStatus(rs.getInt(6));
					professor.setId(rs.getString(7));
					professor.setName(rs.getString(8));
					String address = rs.getString(9);
					if (rs.wasNull()) {
						address = null;
					}
					professor.setAddress(address);
					professor.setOrgAuth(rs.getString(10));
					String orgName = rs.getString(11);
					if (rs.wasNull()) {
						orgName = null;
					}
					professor.setOrgName(orgName);
					String orgId = rs.getString(12);
					if (rs.wasNull()) {
						orgId = null;
					}
					professor.setOrgId(orgId);
					String scorePercent = rs.getString(13);
					if (rs.wasNull()) {
						scorePercent = null;
					}
					professor.setScorePercent(scorePercent);
					professor.setShareId(rs.getLong(14));
					String phone = rs.getString(15);
					if (rs.wasNull()) {
						phone = null;
					}
					professor.setPhone(phone);
					String email = rs.getString(16);
					if (rs.wasNull()) {
						email = null;
					}
					professor.setEmail(email);
					String province = rs.getString(17);
					if (rs.wasNull()) {
						province = null;
					}
					professor.setProvince(province);

					return professor;
				} else {
					return null;
				}
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
	}

	public List<EditProfessor> queryByName(Connection con, String name, int total) throws SQLException {
		int index = 1;
		String sql = "SELECT P.OFFICE,P.DEPARTMENT,P.TITLE,AUTHENTICATION,AUTH_TYPE,P.AUTH_STATUS,P.ID,P.NAME,P.ADDRESS,P.ORG_AUTH,ORGANIZATION.NAME,P.ORG_ID,P.SCORE_PERCENT,P.SHARE_ID "
				+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION ON P.ORG_ID = ORGANIZATION.ID WHERE P.NAME LIKE ? ORDER BY P.NAME ASC LIMIT " + total;
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, name);
			ResultSet rs = ps.executeQuery();
			try {
				List<EditProfessor> editProfessors = new ArrayList<EditProfessor>();
				while (rs.next()) {
					EditProfessor professor = new EditProfessor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String depart = rs.getString(2);
					if (rs.wasNull()) {
						depart = null;
					}
					professor.setDepartment(depart);
					String title = rs.getString(3);
					if (rs.wasNull()) {
						title = null;
					}
					professor.setTitle(title);
					Integer auth = rs.getInt(4);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(5));
					professor.setAuthStatus(rs.getInt(6));
					professor.setId(rs.getString(7));
					professor.setName(rs.getString(8));
					String address = rs.getString(9);
					if (rs.wasNull()) {
						address = null;
					}
					professor.setAddress(address);
					professor.setOrgAuth(rs.getString(10));
					String orgName = rs.getString(11);
					if (rs.wasNull()) {
						orgName = null;
					}
					professor.setOrgName(orgName);
					String orgId = rs.getString(12);
					if (rs.wasNull()) {
						orgId = null;
					}
					professor.setOrgId(orgId);
					String scorePercent = rs.getString(13);
					if (rs.wasNull()) {
						scorePercent = null;
					}
					professor.setScorePercent(scorePercent);
					professor.setShareId(rs.getLong(14));
					editProfessors.add(professor);
				}
				return editProfessors;
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
	}

	/**
	 * 查询专家基本信息+星级+咨询次数的接口
	 * 
	 * @param con
	 * @param id
	 *            专家ID
	 * @return 返回专家基本信息+星级+咨询次数
	 * @throws SQLException
	 */
	public EditProfessor queryEditBaseInfo(Connection con, String id) throws SQLException {
		int index = 1;
		String sql = "SELECT P.OFFICE,P.DEPARTMENT,P.TITLE,AUTHENTICATION,AUTH_TYPE,P.AUTH_STATUS,"
				+ " P.ID,P.NAME,P.ADDRESS,P.STAR_LEVEL,P.CONSULT_COUNT,P.ORG_AUTH,ORGANIZATION.NAME,P.SHARE_ID,P.PHONE,P.EMAIL,P.PROVINCE,P.SEX,P.BIRTHDAY "
				+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION ON P.ORG_ID = ORGANIZATION.ID WHERE P.ID = ?";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, id);
			ResultSet rs = ps.executeQuery();
			try {
				if (rs.next()) {
					EditProfessor professor = new EditProfessor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String depart = rs.getString(2);
					if (rs.wasNull()) {
						depart = null;
					}
					professor.setDepartment(depart);
					String title = rs.getString(3);
					if (rs.wasNull()) {
						title = null;
					}
					professor.setTitle(title);
					Integer auth = rs.getInt(4);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(5));
					professor.setAuthStatus(rs.getInt(6));
					professor.setId(rs.getString(7));
					professor.setName(rs.getString(8));
					String address = rs.getString(9);
					if (rs.wasNull()) {
						address = null;
					}
					professor.setAddress(address);
					professor.setStarLevel(rs.getBigDecimal(10));
					professor.setConsultCount(rs.getInt(11));
					professor.setOrgAuth(rs.getString(12));
					String orgName = rs.getString(13);
					if (rs.wasNull()) {
						orgName = null;
					}
					professor.setOrgName(orgName);
					professor.setShareId(rs.getLong(14));
					professor.setPhone(rs.getString(15));
					professor.setEmail(rs.getString(16));
					professor.setProvince(rs.getString(17));
					professor.setSex(rs.getString(18));
					professor.setBirthday(rs.getString(19));
					return professor;
				} else {
					return null;
				}
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
	}

	@SelectOne
	@Nullable
	public abstract Professor queryOne(Connection con, String id) throws SQLException;

	public Professor query(Connection con, String id) throws SQLException {
		int index = 1;
		String sql = "SELECT P.OFFICE,P.SUBJECT,P.INDUSTRY,P.ADDRESS,P.DEPARTMENT,ORG_ID,P.TITLE,AUTHENTICATION,AUTH_TYPE,"
				+ "P.AUTH_STATUS,P.ID,P.NAME,P.DESCP,P.CREATE_TIME,P.MODIFY_TIME,P.PHONE,P.EMAIL,P.ORG_AUTH,ORGANIZATION.NAME,P.SHARE_ID "
				+ "FROM PROFESSOR P LEFT JOIN ORGANIZATION ON P.ORG_ID = ORGANIZATION.ID WHERE P.ID = ?";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, id);
			ResultSet rs = ps.executeQuery();
			try {
				if (rs.next()) {
					Professor professor = new Professor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String subject = rs.getString(2);
					if (rs.wasNull()) {
						subject = null;
					}
					professor.setSubject(subject);
					String industry = rs.getString(3);
					if (rs.wasNull()) {
						industry = null;
					}
					professor.setIndustry(industry);
					String address = rs.getString(4);
					if (rs.wasNull()) {
						address = null;
					}
					professor.setAddress(address);
					String department = rs.getString(5);
					if (rs.wasNull()) {
						department = null;
					}
					professor.setDepartment(department);
					Organization organization = new Organization();
					String orgId = rs.getString(6);
					if (rs.wasNull()) {
						orgId = null;
					}
					organization.setId(orgId);
					String title = rs.getString(7);
					if (rs.wasNull()) {
						title = null;
					}
					professor.setTitle(title);
					Integer auth = rs.getInt(8);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(9));
					professor.setAuthStatus(rs.getInt(10));
					professor.setId(rs.getString(11));
					professor.setName(rs.getString(12));
					String descp = rs.getString(13);
					if (rs.wasNull()) {
						descp = null;
					}
					professor.setDescp(descp);
					professor.setCreateTime(rs.getString(14));
					professor.setModifyTime(rs.getString(15));
					String phone = rs.getString(16);
					if (rs.wasNull()) {
						phone = null;
					}
					professor.setPhone(phone);
					String email = rs.getString(17);
					if (rs.wasNull()) {
						email = null;
					}
					professor.setEmail(email);
					professor.setOrgAuth(rs.getString(18));
					String orgName = rs.getString(19);
					if (rs.wasNull()) {
						orgName = null;
					}
					organization.setName(orgName);
					professor.setOrganization(organization);
					professor.setShareId(rs.getLong(20));
					return professor;
				} else {
					return null;
				}
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
	}

	public List<Professor> query(Connection con) throws SQLException {
		String sql = "SELECT OFFICE,P.SUBJECT,P.INDUSTRY,P.PROVINCE,P.ADDRESS,DEPARTMENT,ORG_ID,organization.NAME AS ONAME,"
				+ " TITLE,AUTHENTICATION,AUTH_TYPE,P.AUTH_STATUS,P.ID,P.NAME AS PNAME,P.DESCP,P.CREATE_TIME,P.MODIFY_TIME,P.ORG_AUTH,P.SHARE_ID "
				+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION ON P.ORG_ID=organization.ID ORDER BY ONAME,P.ID";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ResultSet rs = ps.executeQuery();
			try {
				List<Professor> professors = new ArrayList<Professor>();
				while (rs.next()) {
					Professor professor = new Professor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String subject = rs.getString(2);
					if (rs.wasNull()) {
						subject = null;
					}
					professor.setSubject(subject);
					String industry = rs.getString(3);
					if (rs.wasNull()) {
						industry = null;
					}
					professor.setIndustry(industry);
					String province = rs.getString(4);
					if (rs.wasNull()) {
						province = null;
					}
					professor.setProvince(province);
					String address = rs.getString(5);
					if (rs.wasNull()) {
						address = null;
					}
					professor.setAddress(address);
					professor.setDepartment(rs.getString(6));
					Organization organization = new Organization();
					String orgId = rs.getString(7);
					if (rs.wasNull()) {
						orgId = null;
					}
					organization.setId(orgId);
					String orgName = rs.getString(8);
					if (rs.wasNull()) {
						orgName = null;
					}
					organization.setName(orgName);
					professor.setOrganization(organization);
					professor.setTitle(rs.getString(9));
					Integer auth = rs.getInt(10);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(11));
					professor.setAuthStatus(rs.getInt(12));
					professor.setId(rs.getString(13));
					professor.setName(rs.getString(14));
					String descp = rs.getString(15);
					if (rs.wasNull()) {
						descp = null;
					}
					professor.setDescp(descp);
					professor.setCreateTime(rs.getString(16));
					professor.setModifyTime(rs.getString(17));
					professor.setOrgAuth(rs.getString(18));
					professor.setShareId(rs.getLong(19));
					professors.add(professor);
				}
				return professors;
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
	}

	public PageQueryResult<EditProfessor> queryPage(Connection con, String orgId, String orgAuth, int pageSize, int pageNo) throws SQLException {
		int total = 0;
		PageQueryResult<EditProfessor> queryResult = new PageQueryResult<EditProfessor>();
		int index = 1;
		boolean hasOrgAuth = null != orgAuth;
		String sql = null;
		sql = " WHERE ORG_ID = ? ";
		if (hasOrgAuth) {
			sql = sql + " AND ORG_AUTH = ?";
		}
		String whereSql = sql;
		sql = "SELECT COUNT(1) FROM PROFESSOR";
		sql = sql + whereSql;
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, orgId);
			if (hasOrgAuth) {
				ps.setString(index++, orgAuth);
			}
			queryResult.setPageSize(pageSize);
			ResultSet rs = ps.executeQuery();
			try {
				rs.next();
				total = rs.getInt(1);
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		queryResult.setTotal(total);
		if (0 == total) {
			queryResult.setPageNo(1);
			queryResult.setData(Collections.<EditProfessor> emptyList());
			return queryResult;
		}
		index = 1;
		boolean firstPage = (1 == pageNo);
		if (firstPage) {
			queryResult.setPageNo(1);
			sql = "SELECT OFFICE,DEPARTMENT,TITLE,AUTHENTICATION,AUTH_TYPE,AUTH_STATUS,ID,NAME,ADDRESS,ORG_AUTH,SHARE_ID FROM PROFESSOR";
			sql = sql + whereSql;
			sql = sql + " ORDER BY AUTH_TYPE DESC,ID";
			sql = sql + " LIMIT " + pageSize;
		} else {
			int pageNum = total / pageSize;
			if (total % pageSize != 0) {
				++pageNum;
			}
			if (pageNo > pageNum) {
				pageNo = pageNum;
			}
			queryResult.setPageNo(pageNo);
			--pageNo;
			int offset = (pageNo * pageSize);
			sql = "SELECT OFFICE,DEPARTMENT,TITLE,AUTHENTICATION,AUTH_TYPE,AUTH_STATUS,ID,NAME,ADDRESS,ORG_AUTH FROM PROFESSOR";
			sql = sql + whereSql;
			sql = sql + " ORDER BY AUTH_TYPE DESC,ID";
			sql = sql + " LIMIT " + pageSize + " OFFSET " + offset;
		}
		ps = con.prepareStatement(sql);
		try {
			ps.setString(index++, orgId);
			if (hasOrgAuth) {
				ps.setString(index++, orgAuth);
			}
			ResultSet rs = ps.executeQuery();
			try {
				List<EditProfessor> professors = new ArrayList<EditProfessor>();
				queryResult.setData(professors);
				int size = 0;
				while ((size < pageSize) && rs.next()) {
					++size;
					EditProfessor professor = new EditProfessor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String depart = rs.getString(2);
					if (rs.wasNull()) {
						depart = null;
					}
					professor.setDepartment(depart);
					String title = rs.getString(3);
					if (rs.wasNull()) {
						title = null;
					}
					professor.setTitle(title);
					Integer auth = rs.getInt(4);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(5));
					professor.setAuthStatus(rs.getInt(6));
					professor.setId(rs.getString(7));
					professor.setName(rs.getString(8));
					String address = rs.getString(9);
					if (rs.wasNull()) {
						address = null;
					}
					professor.setAddress(address);
					professor.setOrgAuth(rs.getString(10));
					professor.setShareId(rs.getLong(11));
					professors.add(professor);
				}
				return queryResult;
			} finally {
				try {
					rs.close();
				} catch (Exception e3) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e4) {
			}
		}
	}

	public List<EditProfessor> queryByOrg(Connection con, String orgId, String orgAuth) throws SQLException {
		int index = 1;
		boolean hasOrgAuth = null != orgAuth;
		StringBuilder sql = new StringBuilder();
		sql.append("SELECT OFFICE,DEPARTMENT,TITLE,AUTHENTICATION,AUTH_TYPE,AUTH_STATUS,ID,NAME,ADDRESS,ORG_AUTH,SHARE_ID FROM PROFESSOR");
		sql.append(" WHERE ORG_ID = ?");
		if (hasOrgAuth) {
			sql.append(" AND ORG_AUTH = ?");
		}
		sql.append(" ORDER BY AUTH_TYPE DESC,ID");
		PreparedStatement ps = con.prepareStatement(sql.toString());
		try {
			ps.setString(index++, orgId);
			if (hasOrgAuth) {
				ps.setString(index++, orgAuth);
			}
			ResultSet rs = ps.executeQuery();
			try {
				List<EditProfessor> professors = new ArrayList<EditProfessor>();
				while (rs.next()) {
					EditProfessor professor = new EditProfessor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String depart = rs.getString(2);
					if (rs.wasNull()) {
						depart = null;
					}
					professor.setDepartment(depart);
					String title = rs.getString(3);
					if (rs.wasNull()) {
						title = null;
					}
					professor.setTitle(title);
					Integer auth = rs.getInt(4);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(5));
					professor.setAuthStatus(rs.getInt(6));
					professor.setId(rs.getString(7));
					professor.setName(rs.getString(8));
					String address = rs.getString(9);
					if (rs.wasNull()) {
						address = null;
					}
					professor.setAddress(address);
					professor.setOrgAuth(rs.getString(10));
					professor.setShareId(rs.getLong(11));
					professors.add(professor);
				}
				return professors;
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
	}

	@LimitQuery
	@OrderBy(cols = {@LimitColumn(value = "lid", column="id", handlerClass = StringHandler.class,asc=false)},value = "")
	@Select(Professor.class)
	@From(Professor.class)
	public abstract List<EditProfessor> orgAuth(Connection con,String orgId, @In(include= false) String[] id, String lid, int rows)throws SQLException;

	@PageQuery
	@Select(Professor.class)
	@From(Professor.class)
	public abstract PageQueryResult<EditProfessor> pqOrgAuth(Connection con, String orgId,@Nullable String orgAuth, int pageSize, int pageNo) throws SQLException;

	/**
	 * 查询所有专家的咨询星级和咨询完成次数
	 * 
	 * @param con
	 * @return
	 * @throws SQLException
	 */
	public List<Professor> queryStar(Connection con) throws SQLException {
		String sql = "SELECT ID,STAR_LEVEL,CONSULT_COUNT FROM PROFESSOR ";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ResultSet rs = ps.executeQuery();
			try {
				List<Professor> professors = new ArrayList<Professor>();
				while (rs.next()) {
					Professor professor = new Professor();
					professor.setId(rs.getString(1));
					professor.setStarLevel(rs.getBigDecimal(2));
					professor.setConsultCount(rs.getInt(3));
					professors.add(professor);
				}
				return professors;
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
	}

	/**
	 * 查询所有专家咨询星级的最大值
	 * 
	 * @param con
	 * @return
	 * @throws SQLException
	 */
	@QueryVal
	@Column(handlerClass = BigDecimalHandler.class, value = { "MAX(STAR_LEVEL)" })
	@From(Professor.class)
	public abstract BigDecimal queryBigStar(Connection con) throws SQLException, JfwBaseException;

	/**
	 * 查询所有专家咨询星级的最小值
	 * 
	 * @param con
	 * @return
	 * @throws SQLException
	 */
	@QueryVal
	@Column(handlerClass = BigDecimalHandler.class, value = { "MIN(STAR_LEVEL)" })
	@From(Professor.class)
	public abstract BigDecimal querySmallStar(Connection con) throws SQLException, JfwBaseException;

	/**
	 * 查询所有专家咨询次数的最大值
	 * 
	 * @param con
	 * @return
	 * @throws SQLException
	 */
	@QueryVal
	@Column(handlerClass = BigDecimalHandler.class, value = { "MAX(CONSULT_COUNT)" })
	@From(Professor.class)
	public abstract BigDecimal queryBigConsult(Connection con) throws SQLException, JfwBaseException;

	/**
	 * 查询所有专家咨询次数的最小值
	 * 
	 * @param con
	 * @return
	 * @throws SQLException
	 */
	@QueryVal
	@Column(handlerClass = BigDecimalHandler.class, value = { "MIN(CONSULT_COUNT)" })
	@From(Professor.class)
	public abstract BigDecimal querySmallConsult(Connection con) throws SQLException, JfwBaseException;

	@Nullable
	@QueryOne
	@From(Professor.class)
	@Select(Professor.class)
	public abstract ProfessorInfo queryInfo(Connection con, String id) throws SQLException;

	@DeleteWith
	@From(Professor.class)
	public abstract int delete(Connection con, String id) throws SQLException;

	@PageSelect
	@Or
	@OrderBy(" ORDER BY ID ")
	public abstract PageQueryResult<Professor> query(Connection con,
			@Nullable @SqlColumn(value = { "ID IN (SELECT PROFESSOR_ID FROM RESEARCH_AREA WHERE CAPTION LIKE ?)",
					"" + "ORG_ID IN(SELECT ID FROM ORGANIZATION WHERE NAME LIKE ?)", "NAME LIKE ?" }, handlerClass = StringHandler.class) String key,
			int pageSize, int pageNo) throws SQLException;

	/**
	 * 首页搜索热门专家
	 * 
	 * @param con
	 * @return 返回8个热门专家，按咨询星级排序
	 * @throws SQLException
	 */
	public PageQueryResult<EditProfessor> queryHot(Connection con, int pageSize, int pageNo) throws SQLException {
		PageQueryResult<EditProfessor> queryResult = new PageQueryResult<EditProfessor>();
		StringBuilder sql = new StringBuilder();
		sql.append(" WHERE AUTH_TYPE = 1 ");
		StringBuilder whereSql = sql;
		sql = new StringBuilder();
		sql.append("SELECT COUNT(1) FROM PROFESSOR");
		if (whereSql.length() > 0) {
			sql.append(whereSql);
		}
		PreparedStatement ps = con.prepareStatement(sql.toString());
		int totalSize = 0;
		try {
			queryResult.setPageSize(pageSize);
			ResultSet rs = ps.executeQuery();
			try {
				rs.next();
				totalSize = rs.getInt(1);
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		queryResult.setTotal(totalSize);
		if (0 == totalSize) {
			queryResult.setPageNo(1);
			queryResult.setData(Collections.<EditProfessor> emptyList());
			return queryResult;
		}
		if (1 == pageNo) {
			queryResult.setPageNo(1);
			sql = new StringBuilder();
			sql.append("SELECT OFFICE,P.ADDRESS,TITLE,AUTHENTICATION,AUTH_TYPE,P.AUTH_STATUS,P.ID,P.NAME,ORG_ID,P.ORG_AUTH,ORGANIZATION.NAME,P.SHARE_ID"
					+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION ON P.ORG_ID=ORGANIZATION.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY SORT_FIRST DESC,STAR_LEVEL DESC,P.ID ");
			sql.append(" LIMIT ").append(pageSize);
		} else {
			int _pageSize = totalSize / pageSize;
			if (totalSize % pageSize != 0) {
				++_pageSize;
			}
			if (pageNo > _pageSize) {
				pageNo = _pageSize;
			}
			queryResult.setPageNo(pageNo);
			--pageNo;
			int offset = (pageNo * pageSize);
			sql = new StringBuilder();
			sql.append("SELECT OFFICE,P.ADDRESS,TITLE,AUTHENTICATION,AUTH_TYPE,P.AUTH_STATUS,P.ID,P.NAME,ORG_ID,P.ORG_AUTH,ORGANIZATION.NAME,P.SHARE_ID"
					+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION ON P.ORG_ID=ORGANIZATION.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY SORT_FIRST DESC,STAR_LEVEL DESC,P.ID ");
			sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
		}
		ps = con.prepareStatement(sql.toString());
		try {
			ResultSet rs = ps.executeQuery();
			try {
				List<EditProfessor> professors = new ArrayList<EditProfessor>();
				queryResult.setData(professors);
				while (rs.next()) {
					EditProfessor professor = new EditProfessor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String address = rs.getString(2);
					if (rs.wasNull()) {
						address = null;
					}
					professor.setAddress(address);
					String title = rs.getString(3);
					if (rs.wasNull()) {
						title = null;
					}
					professor.setTitle(title);
					Integer auth = rs.getInt(4);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(5));
					professor.setAuthStatus(rs.getInt(6));
					professor.setId(rs.getString(7));
					professor.setName(rs.getString(8));
					professor.setOrgId(rs.getString(9));
					professor.setOrgAuth(rs.getString(10));
					String orgName = rs.getString(11);
					if (rs.wasNull()) {
						orgName = null;
					}
					professor.setOrgName(orgName);
					professor.setShareId(rs.getLong(12));
					professors.add(professor);
				}
				return queryResult;
			} finally {
				try {
					rs.close();
				} catch (Exception e3) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e4) {
			}
		}
	}

	/**
	 * APP专家搜索列表分页多条件查询
	 * 
	 * @param con
	 * @param key
	 * @param subject
	 *            学术领域
	 * @param industry
	 *            应用行业
	 * @param province
	 *            所在省份
	 * @param address
	 *            所在城市
	 * @param authentication
	 *            是否认证
	 * @param pageSize
	 * @param pageNo
	 * @return 按搜索条件返回专家列表 按咨询星级排序
	 * @throws SQLException
	 */
	public PageQueryResult<EditProfessor> queryAPP(Connection con, @Nullable String key, String subject, String industry, String province, String address,
			Integer authType, int pageSize, int pageNo) throws SQLException {
		PageQueryResult<EditProfessor> queryResult = new PageQueryResult<EditProfessor>();
		StringBuilder sql = new StringBuilder();
		boolean hasKey = null != key;
		if (hasKey) {
			sql.append(" WHERE (( P.ID IN (SELECT PROFESSOR_ID FROM RESEARCH_AREA WHERE CAPTION LIKE ?)) "
					+ " OR (ORG_ID IN(SELECT ORGANIZATION.ID FROM ORGANIZATION WHERE ORGANIZATION.NAME LIKE ?)) "
					+ " OR ( P.NAME LIKE ?) OR (P.SUBJECT LIKE ?) OR (P.INDUSTRY LIKE ?) OR (P.PROVINCE LIKE ?) OR (P.ADDRESS LIKE ?) "
					+ " OR P.ID IN (SELECT PROFESSOR_ID FROM RESOURCE WHERE RESOURCE_NAME LIKE ?) "
					+ " OR P.ID IN (SELECT OWNER_ID FROM ARTICLE WHERE ARTICLE_TYPE='1' AND ARTICLE_TITLE LIKE ?) "
					+ " OR (P.ID IN (SELECT PROFESSOR_ID FROM PATENT WHERE NAME LIKE ?)) OR (P.DEPARTMENT LIKE ?))");
		}
		boolean hasSubject = null != subject;
		boolean hasIndustry = null != industry;
		boolean hasProvince = null != province;
		boolean hasAddress = null != address;
		boolean hasAuthType = null != authType;
		if (hasSubject) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.SUBJECT LIKE ?)");
		}
		if (hasIndustry) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.INDUSTRY LIKE ?)");
		}
		if (hasProvince) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.PROVINCE LIKE ?)");
		}
		if (hasAddress) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.ADDRESS LIKE ?)");
		}
		if (hasAuthType) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.AUTH_TYPE = ?)");
		}
		StringBuilder whereSql = sql;
		sql = new StringBuilder();
		sql.append("SELECT COUNT(1) FROM PROFESSOR P");
		if (whereSql.length() > 0) {
			sql.append(whereSql);
		}
		PreparedStatement ps = con.prepareStatement(sql.toString());
		int paramIndex = 1;
		int totalSize = 0;
		try {
			if (hasKey) {
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
			}
			if (hasSubject) {
				ps.setString(paramIndex++, subject);
			}
			if (hasIndustry) {
				ps.setString(paramIndex++, industry);
			}
			if (hasProvince) {
				ps.setString(paramIndex++, province);
			}
			if (hasAddress) {
				ps.setString(paramIndex++, address);
			}
			if (hasAuthType) {
				ps.setInt(paramIndex++, authType);
			}
			queryResult.setPageSize(pageSize);
			ResultSet rs = ps.executeQuery();
			try {
				rs.next();
				totalSize = rs.getInt(1);
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		queryResult.setTotal(totalSize);
		if (0 == totalSize) {
			queryResult.setPageNo(1);
			queryResult.setData(Collections.<EditProfessor> emptyList());
			return queryResult;
		}
		paramIndex = 1;
		if (1 == pageNo) {
			queryResult.setPageNo(1);
			sql = new StringBuilder();
			sql.append("SELECT OFFICE,P.ADDRESS,TITLE,AUTHENTICATION,AUTH_TYPE,P.AUTH_STATUS,P.ID,P.NAME,ORG_ID,P.ORG_AUTH,ORGANIZATION.NAME,P.SHARE_ID"
					+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION ON P.ORG_ID=ORGANIZATION.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY SORT_FIRST DESC,STAR_LEVEL DESC,P.ID ");
			sql.append(" LIMIT ").append(pageSize);
		} else {
			int _pageSize = totalSize / pageSize;
			if (totalSize % pageSize != 0) {
				++_pageSize;
			}
			if (pageNo > _pageSize) {
				pageNo = _pageSize;
			}
			queryResult.setPageNo(pageNo);
			--pageNo;
			int offset = (pageNo * pageSize);
			sql = new StringBuilder();
			sql.append("SELECT OFFICE,P.ADDRESS,TITLE,AUTHENTICATION,AUTH_TYPE,P.AUTH_STATUS,P.ID,P.NAME,ORG_ID,P.ORG_AUTH,ORGANIZATION.NAME,P.SHARE_ID"
					+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION ON P.ORG_ID=ORGANIZATION.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY SORT_FIRST DESC,STAR_LEVEL DESC,P.ID ");
			sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
		}
		ps = con.prepareStatement(sql.toString());
		try {
			if (hasKey) {
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
			}
			if (hasSubject) {
				ps.setString(paramIndex++, subject);
			}
			if (hasIndustry) {
				ps.setString(paramIndex++, industry);
			}
			if (hasProvince) {
				ps.setString(paramIndex++, province);
			}
			if (hasAddress) {
				ps.setString(paramIndex++, address);
			}
			if (hasAuthType) {
				ps.setInt(paramIndex++, authType);
			}
			ResultSet rs = ps.executeQuery();
			try {
				List<EditProfessor> professors = new ArrayList<EditProfessor>();
				queryResult.setData(professors);
				while (rs.next()) {
					EditProfessor professor = new EditProfessor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String addr = rs.getString(2);
					if (rs.wasNull()) {
						addr = null;
					}
					professor.setAddress(addr);
					String title = rs.getString(3);
					if (rs.wasNull()) {
						title = null;
					}
					professor.setTitle(title);
					Integer auth = rs.getInt(4);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(5));
					professor.setAuthStatus(rs.getInt(6));
					professor.setId(rs.getString(7));
					professor.setName(rs.getString(8));
					professor.setOrgId(rs.getString(9));
					professor.setOrgAuth(rs.getString(10));
					String orgName = rs.getString(11);
					if (rs.wasNull()) {
						orgName = null;
					}
					professor.setOrgName(orgName);
					professor.setShareId(rs.getLong(12));
					professors.add(professor);
				}
				return queryResult;
			} finally {
				try {
					rs.close();
				} catch (Exception e3) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e4) {
			}
		}
	}

	@LimitQuery
	@OrderBy("ORDER BY sort_desc DESC ")
	public abstract List<SortedPro> indexSearch(Connection con,
											  @SqlColumn(handlerClass = StringHandler.class, value = {"(case when sort_first=0 then '00000' else TO_CHAR(sort_first,'00009') end || TO_CHAR(CEIL(star_level * 100),'0009') || p.id )< ? "}) String sortDesc, @Nullable @GroupSqlColumn(value = {
			"P.NAME LIKE ?", "O.NAME LIKE ?", "P.DEPARTMENT LIKE ?", "P.DESCP LIKE ?", "P.SUBJECT LIKE ?", "P.INDUSTRY LIKE ?", "P.ID IN (SELECT DISTINCT PROFESSOR_ID FROM RESEARCH_AREA WHERE CAPTION LIKE ?)", "P.ID IN (SELECT DISTINCT PROFESSOR_ID FROM RESOURCE WHERE RESOURCE_NAME LIKE ?)", "P.ID IN (SELECT DISTINCT OWNER_ID FROM ARTICLE WHERE ARTICLE_TYPE='1' AND ARTICLE_TITLE LIKE ?)", "P.ID IN (SELECT DISTINCT PROFESSOR_ID FROM PAPER_AUTHOR WHERE PAPER_ID IN (SELECT ID FROM PPAPER WHERE NAME LIKE ?))", "P.ID IN (SELECT DISTINCT PROFESSOR_ID FROM PATENT_AUTHOR WHERE PATENT_ID IN (SELECT ID FROM PPATENT WHERE NAME LIKE ?))"
	}, handlerClass = StringHandler.class, isAnd = false) String key, @Nullable @Like String subject, @Nullable @Like String industry, @Nullable @Like String address, @Nullable Integer authType, int rows) throws SQLException;

	/**
	 * 分页查询专家基础信息，多条件查询专家列表。
	 * 
	 * @param con
	 * @param key
	 *            搜索信息
	 * @param subject
	 *            学术领域
	 * @param industry
	 *            应用行业
	 * @param address
	 *            所在城市
	 * @param pageSize
	 * @param pageNo
	 * @return 返回专家信息列表
	 * @throws SQLException
	 */
	public PageQueryResult<EditProfessor> queryEditBaseInfo(Connection con, @Nullable String key, String subject, String industry, String address,
			Integer authType, int pageSize, int pageNo) throws SQLException {
		
		String fromS=" PROFESSOR P LEFT JOIN ORGANIZATION O ON P.ORG_ID=O.ID ";
		String selectS =" P.OFFICE,P.SUBJECT,P.INDUSTRY,P.ADDRESS,P.DEPARTMENT,O.NAME,P.TITLE,"
					+ "P.AUTHENTICATION,P.AUTH_TYPE,P.AUTH_STATUS,P.ID,P.NAME,P.STAR_LEVEL,P.CONSULT_COUNT,P.ORG_AUTH,P.SHARE_ID,P.SORT_FIRST ";
		PageQueryResult<EditProfessor> queryResult = new PageQueryResult<EditProfessor>();
		StringBuilder sql = new StringBuilder();
		boolean hasKey = null != key;
		if (hasKey) {
			sql.append(" WHERE ((P.NAME LIKE ?) OR")
					.append("(O.NAME LIKE ?) OR")
					.append("( P.DEPARTMENT LIKE ?) OR")
					.append("( P.DESCP LIKE ?) OR")
					.append("( P.SUBJECT LIKE ? ) OR")
					.append("( P.INDUSTRY LIKE ? ) OR")
					.append("(P.ID IN (SELECT DISTINCT PROFESSOR_ID FROM RESEARCH_AREA WHERE CAPTION LIKE ?)) OR")
					.append("(P.ID IN (SELECT DISTINCT PROFESSOR_ID FROM RESOURCE WHERE RESOURCE_NAME LIKE ?)) OR")
					.append("(P.ID IN (SELECT DISTINCT OWNER_ID FROM ARTICLE WHERE ARTICLE_TYPE='1' AND ARTICLE_TITLE LIKE ?)) OR")
					.append("(P.ID IN (SELECT DISTINCT PROFESSOR_ID FROM PAPER_AUTHOR WHERE PAPER_ID IN (SELECT ID FROM PPAPER WHERE NAME LIKE ?))) OR ")
					.append("(P.ID IN (SELECT DISTINCT PROFESSOR_ID FROM PATENT_AUTHOR WHERE PATENT_ID IN (SELECT ID FROM PPATENT WHERE NAME LIKE ?)))")
					.append(") ");
			//
			//
			// sql.append(" WHERE ((P.ID IN (SELECT PROFESSOR_ID FROM
			// RESEARCH_AREA WHERE CAPTION LIKE ?)) "
			// + " OR (ORG_ID IN(SELECT ORGANIZATION.ID FROM ORGANIZATION WHERE
			// ORGANIZATION.NAME LIKE ?)) "
			// + " OR (P.NAME LIKE ?) OR (P.SUBJECT LIKE ?) OR (P.INDUSTRY LIKE
			// ?) OR (P.ADDRESS LIKE ?)"
			// + " OR (P.ID IN (SELECT PROFESSOR_ID FROM RESOURCE WHERE
			// RESOURCE_NAME LIKE ?)) "
			// + " OR (P.ID IN (SELECT PROFESSOR_ID FROM ARTICLE WHERE
			// ARTICLE_TITLE LIKE ?)) "
			// + " OR (P.ID IN (SELECT PROFESSOR_ID FROM PATENT WHERE NAME LIKE
			// ?)) OR (P.DEPARTMENT LIKE ?) )");
		}
		boolean hasSubject = null != subject;
		boolean hasIndustry = null != industry;
		boolean hasAddress = null != address;
		boolean hasAuthType = null != authType;
		if (hasSubject) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.SUBJECT LIKE ?) ");
		}
		if (hasIndustry) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.INDUSTRY LIKE ?) ");
		}
		if (hasAddress) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.ADDRESS LIKE ?) ");
		}
		if (hasAuthType) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.AUTH_TYPE = ?) ");
		}

		StringBuilder whereSql = sql;
		sql = new StringBuilder();
		sql.append("SELECT COUNT(1) FROM").append(fromS);
		if (whereSql.length() > 0) {
			sql.append(whereSql);
		}
		PreparedStatement ps = con.prepareStatement(sql.toString());
		int paramIndex = 1;
		int totalSize = 0;
		try {
			if (hasKey) {
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
			}
			if (hasSubject) {
				ps.setString(paramIndex++, subject);
			}
			if (hasIndustry) {
				ps.setString(paramIndex++, industry);
			}
			if (hasAddress) {
				ps.setString(paramIndex++, address);
			}
			if (hasAuthType) {
				ps.setInt(paramIndex++, authType);
			}
			queryResult.setPageSize(pageSize);
			ResultSet rs = ps.executeQuery();
			try {
				rs.next();
				totalSize = rs.getInt(1);
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		queryResult.setTotal(totalSize);
		if (0 == totalSize) {
			queryResult.setPageNo(1);
			queryResult.setData(Collections.<EditProfessor> emptyList());
			return queryResult;
		}
		paramIndex = 1;
		if (1 == pageNo) {
			queryResult.setPageNo(1);
			sql = new StringBuilder();
			sql.append("SELECT ").append(selectS).append(" FROM ").append(fromS);
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY SORT_FIRST DESC,STAR_LEVEL DESC,P.ID ");
			sql.append(" LIMIT ").append(pageSize);
		} else {
			int _pageSize = totalSize / pageSize;
			if (totalSize % pageSize != 0) {
				++_pageSize;
			}
			if (pageNo > _pageSize) {
				pageNo = _pageSize;
			}
			queryResult.setPageNo(pageNo);
			--pageNo;
			int offset = (pageNo * pageSize);
			sql = new StringBuilder();
			sql.append("SELECT ").append(selectS).append(" FROM ").append(fromS);
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY SORT_FIRST DESC,STAR_LEVEL DESC,P.ID ");
			sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
		}
		ps = con.prepareStatement(sql.toString());
		try {
			if (hasKey) {
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
			}
			if (hasSubject) {
				ps.setString(paramIndex++, subject);
			}
			if (hasIndustry) {
				ps.setString(paramIndex++, industry);
			}
			if (hasAddress) {
				ps.setString(paramIndex++, address);
			}
			if (hasAuthType) {
				ps.setInt(paramIndex++, authType);
			}
			ResultSet rs = ps.executeQuery();
			try {
				List<EditProfessor> professors = new ArrayList<EditProfessor>();
				queryResult.setData(professors);
				while (rs.next()) {
					EditProfessor professor = new EditProfessor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String subj = rs.getString(2);
					if (rs.wasNull()) {
						subj = null;
					}
					professor.setSubject(subj);
					String indus = rs.getString(3);
					if (rs.wasNull()) {
						indus = null;
					}
					professor.setIndustry(indus);
					String addr = rs.getString(4);
					if (rs.wasNull()) {
						addr = null;
					}
					professor.setAddress(addr);
					String depart = rs.getString(5);
					if (rs.wasNull()) {
						depart = null;
					}
					professor.setDepartment(depart);
					String orgName = rs.getString(6);
					if (rs.wasNull()) {
						orgName = null;
					}
					professor.setOrgName(orgName);
					String title = rs.getString(7);
					if (rs.wasNull()) {
						title = null;
					}
					professor.setTitle(title);
					Integer auth = rs.getInt(8);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setAuthType(rs.getInt(9));
					professor.setAuthStatus(rs.getInt(10));
					professor.setId(rs.getString(11));
					professor.setName(rs.getString(12));
					professor.setStarLevel(rs.getBigDecimal(13));
					professor.setConsultCount(rs.getInt(14));
					professor.setOrgAuth(rs.getString(15));
					professor.setShareId(rs.getLong(16));
					professors.add(professor);
				}
				return queryResult;
			} finally {
				try {
					rs.close();
				} catch (Exception e3) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e4) {
			}
		}
	}

	public PageQueryResult<Professor> query(Connection con, @Nullable String key, String subject, String industry, String address, int pageSize, int pageNo)
			throws SQLException {

		PageQueryResult<Professor> queryResult = new PageQueryResult<Professor>();

		StringBuilder sql = new StringBuilder();
		boolean hasKey = null != key;
		if (hasKey) {
			sql.append(" WHERE (( P.ID IN (SELECT PROFESSOR_ID FROM RESEARCH_AREA WHERE CAPTION LIKE ?)) "
					+ " OR (ORG_ID IN(SELECT organization.ID FROM ORGANIZATION WHERE organization.NAME LIKE ?)) "
					+ " OR (P.NAME LIKE ?) OR (P.SUBJECT LIKE ?) OR (P.INDUSTRY LIKE ?) OR (P.ADDRESS LIKE ?))");
		}
		boolean hasSubject = null != subject;
		boolean hasIndustry = null != industry;
		boolean hasAddress = null != address;
		if (hasSubject) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.SUBJECT LIKE ?)");
		}
		if (hasIndustry) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.INDUSTRY LIKE ?)");
		}
		if (hasAddress) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.ADDRESS LIKE ?)");
		}

		StringBuilder whereSql = sql;
		sql = new StringBuilder();
		sql.append("SELECT COUNT(1) FROM PROFESSOR P");
		if (whereSql.length() > 0) {
			sql.append(whereSql);
		}
		PreparedStatement ps = con.prepareStatement(sql.toString());
		int paramIndex = 1;
		int totalSize = 0;
		try {
			if (hasKey) {
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
			}
			if (hasSubject) {
				ps.setString(paramIndex++, subject);
			}
			if (hasIndustry) {
				ps.setString(paramIndex++, industry);
			}
			if (hasAddress) {
				ps.setString(paramIndex++, address);
			}
			queryResult.setPageSize(pageSize);
			ResultSet rs = ps.executeQuery();
			try {
				rs.next();
				totalSize = rs.getInt(1);
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		queryResult.setTotal(totalSize);
		if (0 == totalSize) {
			queryResult.setPageNo(1);
			queryResult.setData(Collections.<Professor> emptyList());
			return queryResult;
		}
		paramIndex = 1;
		if (1 == pageNo) {
			queryResult.setPageNo(1);
			sql = new StringBuilder();
			sql.append("SELECT OFFICE,P.SUBJECT,P.INDUSTRY,P.ADDRESS,DEPARTMENT,ORG_ID,organization.NAME,"
					+ " TITLE,AUTHENTICATION,P.ID,P.NAME,P.DESCP,P.CREATE_TIME,P.MODIFY_TIME,P.ORG_AUTH,P.SHARE_ID "
					+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION ON P.ORG_ID=organization.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY SORT_FIRST DESC,STAR_LEVEL DESC,P.ID ");
			sql.append(" LIMIT ").append(pageSize);
		} else {
			int _pageSize = totalSize / pageSize;
			if (totalSize % pageSize != 0) {
				++_pageSize;
			}
			if (pageNo > _pageSize) {
				pageNo = _pageSize;
			}
			queryResult.setPageNo(pageNo);
			--pageNo;
			int offset = (pageNo * pageSize);
			sql = new StringBuilder();
			sql.append("SELECT OFFICE,P.SUBJECT,P.INDUSTRY,P.ADDRESS,DEPARTMENT,ORG_ID,organization.NAME,"
					+ " TITLE,AUTHENTICATION,P.ID,P.NAME,P.DESCP,P.CREATE_TIME,P.MODIFY_TIME,P.ORG_AUTH,P.SHARE_ID "
					+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION ON P.ORG_ID=organization.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY SORT_FIRST DESC,STAR_LEVEL DESC,P.ID ");
			sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
		}
		ps = con.prepareStatement(sql.toString());
		try {
			if (hasKey) {
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
			}
			if (hasSubject) {
				ps.setString(paramIndex++, subject);
			}
			if (hasIndustry) {
				ps.setString(paramIndex++, industry);
			}
			if (hasAddress) {
				ps.setString(paramIndex++, address);
			}
			ResultSet rs = ps.executeQuery();
			try {
				List<Professor> professors = new ArrayList<Professor>();
				queryResult.setData(professors);
				while (rs.next()) {
					Professor professor = new Professor();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					professor.setOffice(office);
					String subject2 = rs.getString(2);
					if (rs.wasNull()) {
						subject2 = null;
					}
					professor.setSubject(subject2);
					String industry2 = rs.getString(3);
					if (rs.wasNull()) {
						industry2 = null;
					}
					professor.setIndustry(industry2);
					String addr = rs.getString(4);
					if (rs.wasNull()) {
						addr = null;
					}
					professor.setAddress(addr);
					String depart = rs.getString(5);
					if (rs.wasNull()) {
						depart = null;
					}
					professor.setDepartment(depart);
					Organization organization = new Organization();
					String orgId = rs.getString(6);
					if (rs.wasNull()) {
						orgId = null;
					}
					organization.setId(orgId);
					String orgName = rs.getString(7);
					if (rs.wasNull()) {
						orgName = null;
					}
					organization.setName(orgName);
					professor.setOrganization(organization);
					String title = rs.getString(8);
					if (rs.wasNull()) {
						title = null;
					}
					professor.setTitle(title);
					Integer auth = rs.getInt(9);
					if (rs.wasNull()) {
						auth = null;
					}
					professor.setAuthentication(auth);
					professor.setId(rs.getString(10));
					professor.setName(rs.getString(11));
					String descp = rs.getString(12);
					if (rs.wasNull()) {
						descp = null;
					}
					professor.setDescp(descp);
					professor.setCreateTime(rs.getString(13));
					professor.setModifyTime(rs.getString(14));
					professor.setOrgAuth(rs.getString(15));
					professor.setShareId(rs.getLong(16));
					professors.add(professor);
				}
				return queryResult;
			} finally {
				try {
					rs.close();
				} catch (Exception e3) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e4) {
			}
		}
	}

	public PageQueryResult<UserInfo> queryUserInfo(Connection con, String key, String address, Integer authType, String orgName, Integer activeState,
			int pageSize, int pageNo) throws SQLException {
		PageQueryResult<UserInfo> queryResult = new PageQueryResult<UserInfo>();
		StringBuilder sql = new StringBuilder();
		boolean hasKey = null != key;
		if (hasKey) {
			sql.append(" WHERE ( (O.NAME LIKE ?) OR (P.NAME LIKE ?) OR (P.OFFICE LIKE ?) OR (P.TITLE LIKE ?) "
					+ " OR (P.DEPARTMENT LIKE ?) OR (P.PHONE LIKE ?) OR (P.EMAIL LIKE ?) OR (P.ADDRESS LIKE ?) )");
		}
		boolean hasAddress = null != address;
		boolean hasAuthType = null != authType;
		boolean hasOrgName = null != orgName;
		boolean hasActiveState = null != activeState;
		if (hasAddress) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.ADDRESS LIKE ?)");
		}
		if (hasAuthType) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(P.AUTH_TYPE = ?)");
		}
		if (hasOrgName) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(O.NAME LIKE ?)");
		}
		if (hasActiveState) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ");
			if (activeState == 1) {
				sql.append("(L.ACTIVE_TIME IS NOT NULL)");
			} else {
				sql.append("(L.ACTIVE_TIME IS NULL)");
			}
		}

		StringBuilder whereSql = sql;
		sql = new StringBuilder();
		sql.append("SELECT COUNT(1) FROM PROFESSOR P LEFT JOIN ORGANIZATION O ON P.ORG_ID=O.ID LEFT JOIN LUSER L ON P.ID=L.ID ");
		if (whereSql.length() > 0) {
			sql.append(whereSql);
		}
		PreparedStatement ps = con.prepareStatement(sql.toString());
		int paramIndex = 1;
		int totalSize = 0;
		try {
			if (hasKey) {
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
			}
			if (hasAddress) {
				ps.setString(paramIndex++, address);
			}
			if (hasAuthType) {
				ps.setInt(paramIndex++, authType);
			}
			if (hasOrgName) {
				ps.setString(paramIndex++, orgName);
			}
			queryResult.setPageSize(pageSize);
			ResultSet rs = ps.executeQuery();
			try {
				rs.next();
				totalSize = rs.getInt(1);
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		queryResult.setTotal(totalSize);
		if (0 == totalSize) {
			queryResult.setPageNo(1);
			queryResult.setData(Collections.<UserInfo> emptyList());
			return queryResult;
		}
		paramIndex = 1;
		if (1 == pageNo) {
			queryResult.setPageNo(1);
			sql = new StringBuilder();
			sql.append("SELECT OFFICE,P.SUBJECT,P.INDUSTRY,P.ADDRESS,DEPARTMENT,O.NAME,TITLE,AUTHENTICATION,"
					+ " P.AUTH_TYPE,P.AUTH_STATUS,P.ID,P.NAME,P.EMAIL,P.PHONE,L.ACTIVE_TIME,P.ORG_AUTH,P.SHARE_ID "
					+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION O ON P.ORG_ID=O.ID LEFT JOIN LUSER L ON P.ID=L.ID");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY P.CREATE_TIME DESC,P.ID ");
			sql.append(" LIMIT ").append(pageSize);
		} else {
			int _pageSize = totalSize / pageSize;
			if (totalSize % pageSize != 0) {
				++_pageSize;
			}
			if (pageNo > _pageSize) {
				pageNo = _pageSize;
			}
			queryResult.setPageNo(pageNo);
			--pageNo;
			int offset = (pageNo * pageSize);
			sql = new StringBuilder();
			sql.append("SELECT OFFICE,P.SUBJECT,P.INDUSTRY,P.ADDRESS,DEPARTMENT,O.NAME,TITLE,AUTHENTICATION,"
					+ " P.AUTH_TYPE,P.AUTH_STATUS,P.ID,P.NAME,P.EMAIL,P.PHONE,L.ACTIVE_TIME,P.ORG_AUTH,P.SHARE_ID "
					+ " FROM PROFESSOR P LEFT JOIN ORGANIZATION O ON P.ORG_ID=O.ID LEFT JOIN LUSER L ON P.ID=L.ID");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY P.CREATE_TIME DESC,P.ID ");
			sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(offset);
		}
		ps = con.prepareStatement(sql.toString());
		try {
			if (hasKey) {
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
			}
			if (hasAddress) {
				ps.setString(paramIndex++, address);
			}
			if (hasAuthType) {
				ps.setInt(paramIndex++, authType);
			}
			if (hasOrgName) {
				ps.setString(paramIndex++, orgName);
			}
			ResultSet rs = ps.executeQuery();
			try {
				List<UserInfo> userInfos = new ArrayList<UserInfo>();
				queryResult.setData(userInfos);
				while (rs.next()) {
					UserInfo userInfo = new UserInfo();
					String office = rs.getString(1);
					if (rs.wasNull()) {
						office = null;
					}
					userInfo.setOffice(office);
					String subj = rs.getString(2);
					if (rs.wasNull()) {
						subj = null;
					}
					userInfo.setSubject(subj);
					String indus = rs.getString(3);
					if (rs.wasNull()) {
						indus = null;
					}
					userInfo.setIndustry(indus);
					String addr = rs.getString(4);
					if (rs.wasNull()) {
						addr = null;
					}
					userInfo.setAddress(addr);
					String depart = rs.getString(5);
					if (rs.wasNull()) {
						depart = null;
					}
					userInfo.setDepartment(depart);
					String org = rs.getString(6);
					if (rs.wasNull()) {
						org = null;
					}
					userInfo.setOrgName(org);
					String title = rs.getString(7);
					if (rs.wasNull()) {
						title = null;
					}
					userInfo.setTitle(title);
					Integer auth = rs.getInt(8);
					if (rs.wasNull()) {
						auth = null;
					}
					userInfo.setAuthentication(auth);
					userInfo.setAuthType(rs.getInt(9));
					userInfo.setAuthStatus(rs.getInt(10));
					userInfo.setId(rs.getString(11));
					userInfo.setName(rs.getString(12));
//					userInfo.setEmail(rs.getString(13));
//					userInfo.setPhone(rs.getString(14));
					userInfo.setActiveTime(rs.getString(15));
					userInfo.setActiveState(!(rs.getString(15) == null));
					userInfo.setOrgAuth(rs.getString(16));
					userInfo.setShareId(rs.getLong(17));
					userInfos.add(userInfo);
				}
				return queryResult;
			} finally {
				try {
					rs.close();
				} catch (Exception e3) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e4) {
			}
		}
	}

	public String[] queryProfessorIdWithSameKeyWord(Connection con, String id, int rows) throws SQLException {
		List<String> ret = new ArrayList<String>();
		String sql = "select id,count(1) sort from pro_key_word where kw in (select kw from pro_key_word where id=?) and id <>? group by id order by sort desc limit ?";
		PreparedStatement ps = con.prepareStatement(sql);
		try {
			ps.setString(1, id);
			ps.setString(2, id);
			ps.setInt(3, rows);
			ResultSet rs = ps.executeQuery();
			try {
				while (rs.next()) {
					ret.add(rs.getString(1));
				}
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		return ret.isEmpty() ? null : ret.toArray(new String[ret.size()]);
	}

	@SelectList
	public abstract List<Professor> query(Connection con, @In String[] id) throws SQLException;

	@QueryList
	@Select(Professor.class)
	@From(Professor.class)
	public abstract List<EditProfessor> queryPro(Connection con, @In String[] id) throws SQLException;

	@UpdateWith
	@Where("orgAuth ='1'")
	@SetSentence("orgAuth='0'")
	@From(Professor.class)
	public abstract int cancelOrgAuth(Connection con,String id)throws SQLException;
}
