package com.ekexiu.portal.job;

import com.ekexiu.portal.pojo.DataDictCode;
import com.ekexiu.portal.service.DataDictService;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Bean;
import org.jfw.util.jdbc.JdbcUtil;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

@Bean
public class HotKeyJobEntry extends DictTaskJobEntry implements Runnable {
	private String taskTime = "2:00:00";
	private long delayTime = 86400000L;
	@Autowrie("dataSource")
	private DataSource dataSource;
	@Autowrie
	private DataDictService dataDictService;

	public String getTaskTime() {
		return taskTime;
	}

	public void setTaskTime(String taskTime) {
		this.taskTime = taskTime;
	}

	public long getDelayTime() {
		return delayTime;
	}

	public void setDelayTime(long delayTime) {
		this.delayTime = delayTime;
	}

	public DataSource getDataSource() {
		return dataSource;
	}

	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	public DataDictService getDataDictService() {
		return dataDictService;
	}

	public void setDataDictService(DataDictService dataDictService) {
		this.dataDictService = dataDictService;
	}

	private void addEntry(String caption, Map<String, DataDictCode> map) {
		DataDictCode item = map.get(caption);
		if (item == null) {
			item = new DataDictCode();
			item.setCaption(caption);
			item.setCount(1);
			map.put(caption, item);
		} else {
			item.setCount(1 + item.getCount());
		}
	}

	private void addEntrys(Connection con, String sql, Map<String, DataDictCode> map) throws SQLException {
		Statement st = con.createStatement();
		try {
			ResultSet rs = st.executeQuery(sql);
			try {
				while (rs.next()) {
					addEntry(rs.getString(1), map);
				}
			} finally {
				JdbcUtil.close(rs);
			}
		} finally {
			JdbcUtil.close(st);
		}
	}

	@Override
	public void run() {

		Map<String, DataDictCode> tmpMap = new HashMap<String, DataDictCode>();
		try {
			Connection con = this.dataSource.getConnection();
			try {
				addEntrys(con, "SELECT KW FROM ART_KEY_WORD", tmpMap);
				addEntrys(con, "SELECT KW FROM PRO_KEY_WORD", tmpMap);
				addEntrys(con, "SELECT KW FROM ORG_KEY_WORD", tmpMap);
				addEntrys(con, "SELECT KW FROM RES_KEY_WORD", tmpMap);
				addEntrys(con, "SELECT KW FROM PAP_KEY_WORD", tmpMap);
				addEntrys(con, "SELECT KW FROM PAT_KEY_WORD", tmpMap);
				addEntrys(con, "SELECT KW FROM TEAM_KEY_WORD", tmpMap);
				ArrayList<DataDictCode> list = new ArrayList<DataDictCode>(tmpMap.size());
				list.addAll(tmpMap.values());
				Collections.sort(list, new Comparator<DataDictCode>() {
					public int compare(DataDictCode o1, DataDictCode o2) {
						long l = o2.getCount() - o1.getCount();
						if (l == 0)
							return 0;
						else if (l > 0)
							return 1;
						else
							return -1;
					}
				});
				this.dataDictService.setHotKey(list);
			} catch (Exception e) {
					e.printStackTrace();
			} finally {
				JdbcUtil.close(con);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
