package com.ekexiu.portal.msg;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.ListIterator;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.util.DateUtil;
import org.jfw.util.PageQueryResult;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.jdbc.PreparedStatementConfig;
import org.jfw.util.jdbc.ResultSetExtractor;
import org.jfw.util.json.JsonService;

import com.ekexiu.portal.util.WebMsgSocketServlet;
import com.ekexiu.push.service.PushService;

@Path("/webMsg")
public class Service {

	public static final List<ActorItem> EMPTY_ACTORS = Collections.<ActorItem> emptyList();

	public static final int ERROR_OWNER_OR_ACTOR = JfwBaseException.MIN_LOGIC_ERROR_CODE + 1;

	@Autowrie
	private WebMsgDao webMsgDao;
	
	@Autowrie
	private PushService pushService;
	
	

	public PushService getPushService() {
		return pushService;
	}

	public void setPushService(PushService pushService) {
		this.pushService = pushService;
	}

	public WebMsgDao getWebMsgDao() {
		return webMsgDao;
	}

	public void setWebMsgDao(WebMsgDao webMsgDao) {
		this.webMsgDao = webMsgDao;
	}

	public static ActorItem build(WebMsgIdx wmi, String owner, SimpleDateFormat sf) {
		ActorItem item = new ActorItem();
		item.setCnt(wmi.getLastCnt());
		item.setTime(wmi.getLastTime());
		item.setTimeStr(sf.format(new Date(wmi.getLastTime())));
		String actor = wmi.getActor();
		if (owner.equals(actor)) {
			item.setId(wmi.getOwner());
			item.setNum(wmi.getNumOfActor());
		} else {
			item.setId(actor);
			item.setNum(wmi.getNumOfOwner());
		}

		return item;
	}

	public static List<ActorItem> build(List<WebMsgIdx> list, String owner) {
		if (list == null || list.isEmpty())
			return EMPTY_ACTORS;

		SimpleDateFormat sf = new SimpleDateFormat(DateUtil.TIMESTAMP_FORMAT);
		List<ActorItem> ret = new ArrayList<ActorItem>(list.size());
		for (ListIterator<WebMsgIdx> it = list.listIterator(); it.hasNext();) {
			ret.add(build(it.next(), owner, sf));
		}
		return ret;
	}

	public static void checkActors(String id, String id2) throws JfwBaseException {
		if (id.compareTo(id2) == 0)
			throw new JfwBaseException(ERROR_OWNER_OR_ACTOR, "消息的参与人不合法");
	}

	/**
	 * 查询指定用户的未删除会话
	 * 
	 * @param con
	 * @param id
	 *            用户ID
	 * @return
	 * @throws SQLException
	 */
	@Get
	@Path("/idx/qm")
	public List<ActorItem> query(@JdbcConn Connection con, String id) throws SQLException {
		List<WebMsgIdx> list = webMsgDao.query(con, id);
		return build(list, id);
	}

	/**
	 * 查询指定的未删除会话
	 * 
	 * @param con
	 * @param owner
	 *            会话所有人
	 * @param actor
	 *            会话参与人
	 * @return
	 * @throws SQLException
	 */
	@Get
	@Path("/idx/qo")
	public ActorItem query(@JdbcConn Connection con, String owner, String actor) throws SQLException {
		WebMsgIdx wmi = null;
		if (owner.compareTo(actor) < 0) {
			wmi = webMsgDao.query(con, owner, actor);
		} else {
			wmi = webMsgDao.query(con, actor, owner);
		}
		SimpleDateFormat sf = new SimpleDateFormat(DateUtil.TIMESTAMP_FORMAT);
		if (wmi != null) {
			return build(wmi, owner, sf);
		}
		return null;
	}

	/**
	 * 查询指定用户的未删除会话(分页,最后活动时间倒序）
	 * 
	 * @param con
	 * @param id
	 *            用户ID
	 * @return
	 * @throws SQLException
	 */
	@Get
	@Path("/idx/pq")
	public PageQueryResult<ActorItem> query(@JdbcConn Connection con, String id, @DefaultValue("1") int pageNo, @DefaultValue("10") int pageSize)
			throws SQLException {
		PageQueryResult<WebMsgIdx> wmis = this.webMsgDao.query(con, id, pageSize, pageNo);
		PageQueryResult<ActorItem> ret = new PageQueryResult<ActorItem>();
		ret.setPageNo(wmis.getPageNo());
		ret.setPageSize(wmis.getPageSize());
		ret.setTotal(wmis.getTotal());
		ret.setData(build(wmis.getData(), id));
		return ret;
	}

	/**
	 * 查询指定用户的未读消息数
	 * 
	 * @param con
	 * @param id
	 *            用户id
	 * @return
	 * @throws SQLException
	 */
	@Get
	@Path("/unReadedCount")
	public Integer queryUnReadedCount(@JdbcConn Connection con, final String id) throws SQLException {
		return JdbcUtil.query(con,
				"SELECT NUM_OF_OWNER FROM WEB_MSG_IDX WHERE OWNER =? AND SHOW_OF_OWNER='1' UNION ALL SELECT NUM_OF_ACTOR FROM WEB_MSG_IDX WHERE ACTOR =? AND SHOW_OF_ACTOR='1' ",
				new ResultSetExtractor<Integer>() {
					@Override
					public Integer extractData(ResultSet rs) throws SQLException {
						int ret = 0;
						while (rs.next()) {
							ret += rs.getInt(1);
						}
						return ret;
					}
				}, new PreparedStatementConfig() {
					@Override
					public void config(PreparedStatement ps) throws SQLException {
						ps.setString(1, id);
						ps.setString(2, id);
					}
				});
	}

	/**
	 * 发送消息
	 * 
	 * @param con
	 * @param sender
	 *            发送人ID
	 * @param reciver
	 *            接收人ID
	 * @param cnt
	 *            内容
	 * @return 发送时间（YYYYMMDDHH24MISSZZZ)
	 * @throws SQLException
	 * @throws JfwBaseException
	 */
	@Post
	@Path
	public String send(@JdbcConn Connection con, String sender, String reciver, String cnt) throws SQLException, JfwBaseException {
		checkActors(sender, reciver);
		boolean ownerSend = true;
		String owner = sender;
		String actor = reciver;
		boolean next = true;
		if (sender.compareTo(reciver) > 0) {
			ownerSend = false;
			owner = reciver;
			actor = sender;
		}
		WebMsgCnt wmc = new WebMsgCnt();
		wmc.setCnt(cnt);
		wmc.setReaded(false);
		wmc.setReciver(reciver);
		wmc.setSender(sender);
		WebMsgIdx wmi = null;

		while (next) {
			try {
				long time = System.currentTimeMillis();
				wmc.setSendTime(DateUtil.formatTimeStamp(time));
				webMsgDao.insert(con, wmc);
				int numOfUpdate = 0;
				if (ownerSend) {
					numOfUpdate = webMsgDao.incActorNum(con, owner, actor, cnt, time);
				} else {
					numOfUpdate = webMsgDao.incOwnerNum(con, owner, actor, cnt, time);
				}
				if (numOfUpdate == 0) {
					if (wmi == null) {
						wmi = new WebMsgIdx();
						wmi.setActor(actor);
						wmi.setLastCnt(cnt);

						wmi.setNumOfActor(ownerSend ? 1 : 0);
						wmi.setNumOfOwner(ownerSend ? 0 : 1);
						wmi.setOwner(owner);
						wmi.setShowOfActor(true);
						wmi.setShowOfOwner(true);
					}
					wmi.setLastTime(time);
					webMsgDao.insert(con, wmi);
				}
				con.commit();
				next = false;
				WebMsgSocketServlet.sendMessage(reciver, JsonService.toJson(wmc));
				try{
					pushService.pushWithAlias("通知","您收到了一条消息", "msgsend", reciver);
				}catch(Throwable thr){}
			} catch (SQLException e) {
				if ("23505".equals(e.getSQLState())) {
					con.rollback();
				}
				throw e;
			}
		}
		return wmc.getSendTime();
	}

	/**
	 * 查询消息内容
	 * 
	 * @param con
	 * @param sender
	 *            发送人ID
	 * @param reciver
	 *            接收人ID
	 * @param bt
	 *            >=开始时间（YYYYMMDDHH24MISSZZZ)
	 * @param et
	 *            <=结束时间（YYYYMMDDHH24MISSZZZ)
	 * @param both
	 *            是否包含反向消息
	 * @return
	 * @throws SQLException
	 */
	@Get
	@Path("/cnt/load")
	public List<WebMsgCnt> load(@JdbcConn Connection con, String sender, String reciver, @Nullable String bt, @Nullable String et,
			@DefaultValue("false") boolean both) throws SQLException {
		if (both) {
			return webMsgDao.queryAll(con, sender, reciver, bt, et);
		} else {
			return webMsgDao.query(con, sender, reciver, bt, et);
		}
	}

	/**
	 * 倒序查询消息内容
	 * 
	 * @param con
	 * @param sender
	 *            发送人
	 * @param reciver
	 *            接收人
	 * @param sendTime
	 *            <最后发送时间
	 * @param rows
	 *            返加数据最大条数
	 * @param both
	 *            是否包含反向消息
	 * @return
	 * @throws SQLException
	 */
	@Get
	@Path("/cnt/deload")
	public List<WebMsgCnt> deLoad(@JdbcConn Connection con, String sender, String reciver, @Nullable String sendTime, @DefaultValue("10") int rows,
			@DefaultValue("false") boolean both) throws SQLException {
		return both ? webMsgDao.queryAll4Desc(con, sender, reciver, sendTime, rows) : webMsgDao.query4Desc(con, sender, reciver, sendTime, rows);
	}
/**
 * 置消息为已读
 * @param con
 * @param sender
 * @param reciver
 * @param time  <=send_time 
 * @throws SQLException
 * @throws JfwBaseException
 */
	@Post
	@Path("/readed")
	public void readed(@JdbcConn(true) Connection con, String sender, String reciver, String time) throws SQLException, JfwBaseException {
		checkActors(sender, reciver);
		int num = webMsgDao.readed(con, sender, reciver, time);
		if (num > 0) {
			String owner = sender;
			String actor = reciver;
			boolean ownerSend = true;
			if (sender.compareTo(reciver) > 0) {
				owner = reciver;
				actor = sender;
				ownerSend = false;
			}
			if (ownerSend) {
				webMsgDao.decActorNum(con, owner, actor, num);
			} else {
				webMsgDao.decOwnerNum(con, owner, actor, num);
			}
		}
	}

	/**
	 * 删除会话
	 * @param con
	 * @param owner  要删除的会话的参与人ID
	 * @param actor    另一个参与人ID
	 * @throws SQLException
	 * @throws JfwBaseException
	 */
	@Post
	@Path("/disable/show")
	public void disableShow(@JdbcConn(true) Connection con, String owner, String actor) throws SQLException, JfwBaseException {
		checkActors(owner, actor);
		if (owner.compareTo(actor) < 0) {
			webMsgDao.disableOwnerShow(con, owner, actor);
		} else {
			webMsgDao.disableActorShow(con, actor, owner);
		}
	}
}
