package com.ekexiu.portal.oauth.weixin;

import java.io.IOException;

import org.jfw.apt.annotation.Bean;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.json.JsonService;

import com.ekexiu.portal.oauth.OAuthHandler;
import com.ekexiu.portal.oauth.OAuthUser;
import com.ekexiu.portal.util.HttpUtil;

@Bean
public class WeiXinHandler implements OAuthHandler {
	public static final String OAUTH_TYPE = "weixin";

	/**
	 * 在微信上注册的APPID
	 */
	private String appid;
	/**
	 * 在微信上注册的APPID 对应的 APPSECRET
	 */
	private String appsecret;
	/**
	 * 微信请求OAUTH登录的URL
	 */
	private String codeUrl = "https://open.weixin.qq.com/connect/qrconnect";
	/**
	 * 微信登录授权后的跳转面页
	 */
	private String redirectUri;
	/**
	 * 微信请求OAUTH AccessToken的URL
	 */
	private String accessTokenUrl = "https://api.weixin.qq.com/sns/oauth2/access_token";

	public String getAccessTokenUrl() {
		return accessTokenUrl;
	}

	public void setAccessTokenUrl(String accessTokenUrl) {
		this.accessTokenUrl = accessTokenUrl;
	}

	public String getRedirectUri() {
		return redirectUri;
	}

	public void setRedirectUri(String redirectUri) {
		this.redirectUri = redirectUri;
	}

	public String getAppid() {
		return appid;
	}

	public void setAppid(String appid) {
		this.appid = appid;
	}

	public String getAppsecret() {
		return appsecret;
	}

	public void setAppsecret(String appsecret) {
		this.appsecret = appsecret;
	}

	public String getCodeUrl() {
		return codeUrl;
	}

	public void setCodeUrl(String codeUrl) {
		this.codeUrl = codeUrl;
	}

	@Override
	public String getType() {
		return OAUTH_TYPE;
	}

	@Override
	public String getLoginUrl() {
		StringBuilder sb = new StringBuilder();
		sb.append(this.codeUrl).append("?appid=").append(this.appid).append("&redirect_uri=").append(this.redirectUri)
				.append("&response_type=code&scope=snsapi_login&state=weixin#wechat_redirect");
		return sb.toString();
	}

	@Override
	public OAuthUser login(String code) throws JfwBaseException {
        AccessTokenResponse atr;
		try {
			atr = HttpUtil.loadJsonObject(this.buildAccessTokenURL(code),AccessTokenResponse.class);
		} catch (IOException e) {
			throw new JfwBaseException(91,e);
		}
        if(null!= atr.getErrcode()) throw new JfwBaseException(0,"访问微信access_token error:"+JsonService.toJson(atr));
		OAuthUser user = new OAuthUser();
		user.setType(this.getType());
		user.setOpenId(atr.getOpenid());
		user.setAccessToken(atr.getAccess_token());
		user.setExpiresIn(atr.getExpires_in());
		user.setScope(atr.getScope());
		user.setRefreshToken(atr.getRefresh_token());
		return user;
	}

	private String buildAccessTokenURL(String code) {
		StringBuilder sb = new StringBuilder();
		sb.append(this.accessTokenUrl).append("?appid=").append(this.appid)
		.append("&secret=").append(this.appsecret)
		.append("&code=").append(code)
		.append("&grant_type=authorization_code");
		return sb.toString();
	}

}
