package com.ekexiu.portal.service;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.util.StringUtil;

import com.ekexiu.portal.dao.LeaveWordDao;
import com.ekexiu.portal.dao.ProfessorDao;
import com.ekexiu.portal.po.LeaveWord;
import com.ekexiu.portal.pojo.EditProfessor;

@Path("/leaveWord")
public class LeaveWordService {
	public static final String MAX_CREATETIME="9";
	
	public static final String ARTICLE="1";
	public static final String PAPER="2";
	public static final String PATENT="3";
	

	@Autowrie
	private LeaveWordDao leaveWordDao;
	@Autowrie
	private ProfessorDao professorDao;
	@Autowrie
	private ImageService imageService;

	public LeaveWordDao getLeaveWordDao() {
		return leaveWordDao;
	}

	public void setLeaveWordDao(LeaveWordDao leaveWordDao) {
		this.leaveWordDao = leaveWordDao;
	}
	
	public ProfessorDao getProfessorDao() {
		return professorDao;
	}

	public void setProfessorDao(ProfessorDao professorDao) {
		this.professorDao = professorDao;
	}

	public ImageService getImageService() {
		return imageService;
	}

	public void setImageService(ImageService imageService) {
		this.imageService = imageService;
	}

	@Post
	@Path
	public String insert(@JdbcConn(true) Connection con,String content,String sender,String articleId) throws SQLException{
		String id = StringUtil.buildUUID();
		LeaveWord word = new LeaveWord();
		word.setId(id);
		word.setOrderKey(System.nanoTime());
		word.setObjId(articleId);
		word.setCategory(ARTICLE);
		word.setContent(content);
		word.setSender(sender);
		this.leaveWordDao.insert(con, word);
		return id;
	}
	
	@Post
	@Path("/paper")
	public String paper(@JdbcConn(true) Connection con,String paperId,String content,String sender) throws SQLException{
		String id = StringUtil.buildUUID();
		LeaveWord word = new LeaveWord();
		word.setId(id);
		word.setOrderKey(System.nanoTime());
		word.setObjId(paperId);
		word.setCategory(PAPER);
		word.setContent(content);
		word.setSender(sender);
		this.leaveWordDao.insert(con, word);
		return id;
	}
	
	@Post
	@Path("/patent")
	public String patent(@JdbcConn(true) Connection con,String patentId,String content,String sender) throws SQLException{
		String id = StringUtil.buildUUID();
		LeaveWord word = new LeaveWord();
		word.setId(id);
		word.setOrderKey(System.nanoTime());
		word.setObjId(patentId);
		word.setCategory(PATENT);
		word.setContent(content);
		word.setSender(sender);
		this.leaveWordDao.insert(con, word);
		return id;
	}
	
	@Get
	@Path("/lwCount")
	public int queryCount(@JdbcConn Connection con,String articleId)throws SQLException{
		return this.leaveWordDao.queryCount(con, articleId,ARTICLE);
	}
	@Get
	@Path("/lwCount/paper")
	public int queryCountByPaper(@JdbcConn Connection con,String paperId)throws SQLException{
		return this.leaveWordDao.queryCount(con, paperId,PAPER);
	}
	@Get
	@Path("/lwCount/patent")
	public int queryCountByPatent(@JdbcConn Connection con,String patentId)throws SQLException{
		return this.leaveWordDao.queryCount(con, patentId,PATENT);
	}

	
	
	
	@Get
	@Path("/ql")
	public List<LeaveWord> query(@JdbcConn Connection con,String articleId,@DefaultValue("com.ekexiu.portal.service.LeaveWordService.MAX_CREATETIME")  String createTime,@DefaultValue("Long.MAX_VALUE") long orderKey,@DefaultValue("10") int rows) throws SQLException{
		List<LeaveWord> leaveWords = this.leaveWordDao.query(con, articleId,ARTICLE, createTime, orderKey, rows);
		if(!leaveWords.isEmpty()){
			for (LeaveWord leaveWord : leaveWords) {
				EditProfessor professor = this.professorDao.queryBaseInfo(con, leaveWord.getSender());
				if(professor != null){
					professor.setHasHeadImage(this.imageService.hasProfessorImage(leaveWord.getSender()));
				}
				leaveWord.setProfessor(professor);
			}
		}
		return leaveWords;
	}
	
	@Get
	@Path("/ql/paper")
	public List<LeaveWord> queryWithPaper(@JdbcConn Connection con,String paperId,@DefaultValue("com.ekexiu.portal.service.LeaveWordService.MAX_CREATETIME")  String createTime,@DefaultValue("Long.MAX_VALUE") long orderKey,@DefaultValue("10") int rows) throws SQLException{
		List<LeaveWord> leaveWords = this.leaveWordDao.query(con, paperId,PAPER, createTime, orderKey, rows);
		if(!leaveWords.isEmpty()){
			for (LeaveWord leaveWord : leaveWords) {
				EditProfessor professor = this.professorDao.queryBaseInfo(con, leaveWord.getSender());
				if(professor != null){
					professor.setHasHeadImage(this.imageService.hasProfessorImage(leaveWord.getSender()));
				}
				leaveWord.setProfessor(professor);
			}
		}
		return leaveWords;
	}	@Get
	@Path("/ql/patent")
	public List<LeaveWord> queryWithPatent(@JdbcConn Connection con,String patentId,@DefaultValue("com.ekexiu.portal.service.LeaveWordService.MAX_CREATETIME")  String createTime,@DefaultValue("Long.MAX_VALUE") long orderKey,@DefaultValue("10") int rows) throws SQLException{
		List<LeaveWord> leaveWords = this.leaveWordDao.query(con, patentId,PATENT, createTime, orderKey, rows);
		if(!leaveWords.isEmpty()){
			for (LeaveWord leaveWord : leaveWords) {
				EditProfessor professor = this.professorDao.queryBaseInfo(con, leaveWord.getSender());
				if(professor != null){
					professor.setHasHeadImage(this.imageService.hasProfessorImage(leaveWord.getSender()));
				}
				leaveWord.setProfessor(professor);
			}
		}
		return leaveWords;
	}
	
	@Post
	@Path("/delete")
	public void delete(@JdbcConn(true) Connection con,String id)throws SQLException{
		this.leaveWordDao.delete(con, id);
	}
}
