package com.ekexiu.portal.util;

import com.ekexiu.portal.po.*;
import com.itextpdf.io.source.ByteArrayOutputStream;
import com.itextpdf.kernel.color.Color;
import com.itextpdf.kernel.color.DeviceRgb;
import com.itextpdf.kernel.events.Event;
import com.itextpdf.kernel.events.IEventHandler;
import com.itextpdf.kernel.events.PdfDocumentEvent;
import com.itextpdf.kernel.font.PdfFont;
import com.itextpdf.kernel.font.PdfFontFactory;
import com.itextpdf.kernel.geom.PageSize;
import com.itextpdf.kernel.geom.Rectangle;
import com.itextpdf.kernel.pdf.PdfDocument;
import com.itextpdf.kernel.pdf.PdfPage;
import com.itextpdf.kernel.pdf.PdfWriter;
import com.itextpdf.kernel.pdf.canvas.PdfCanvas;
import com.itextpdf.layout.Document;
import com.itextpdf.layout.border.SolidBorder;
import com.itextpdf.layout.element.*;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.jdbc.PreparedStatementConfig;
import org.jfw.util.jdbc.ResultSetExtractor;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collections;
import java.util.Comparator;

/**
 * Servlet implementation class ProfessorPdfServlet
 */
public class ProfessorPdfServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	public static final Color RGB3 = new DeviceRgb(0x33, 0x33, 0x33);
	public static final Color RGB9 = new DeviceRgb(0x77, 0x77, 0x77);
	public static final Color RGBC = new DeviceRgb(0xf2, 0xf2, 0xf2);

	public static final float TITLE_FONT_SIZE = 12f;
	public static final float TEXT_FONT_SIZE = 10f;
	public static final float LINE_INTERVAL_L = 2.15f;
	public static final float LINE_INTERVAL_M = 2.15f;

	private DataSource dataSource;

	public ProfessorPdfServlet() {
		super();
	}

	private void sortPaperAndPatent(ViewBean vb) {
		final String name = vb.getName() + ",";

		java.util.List<Ppaper> ppas = vb.getPapers();
		if (ppas != null && ppas.size() > 1) {
			Collections.sort(ppas, new Comparator<Ppaper>() {
				@Override
				public int compare(Ppaper o1, Ppaper o2) {
					boolean f1 = (o1.getAuthors().indexOf(',') < 0) || o1.getAuthors().startsWith(name);
					boolean f2 = (o2.getAuthors().indexOf(',') < 0) || o2.getAuthors().startsWith(name);
					if (f1 == f2) {
						if (o1.getPubDay() == null || o2.getPubDay() == null) {
							return 0;
						}else {
							return -(o1.getPubDay().compareTo(o2.getPubDay()));
						}
					} else {
						return f1 ? -1 : 1;
					}
				}
			});
		}

		java.util.List<Ppatent> ppts = vb.getPatents();
		if (ppts != null && ppts.size() > 1) {
			Collections.sort(ppts, new Comparator<Ppatent>() {
				@Override
				public int compare(Ppatent o1, Ppatent o2) {
					boolean f1 = (o1.getAuthors().indexOf(',') < 0) || o1.getAuthors().startsWith(name);
					boolean f2 = (o2.getAuthors().indexOf(',') < 0) || o2.getAuthors().startsWith(name);
					if (f1 == f2) {
						return -(o1.getPubDay().compareTo(o2.getPubDay()));
					} else {
						return f1 ? -1 : 1;
					}
				}
			});
		}

	}

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String id = request.getParameter("id");
		boolean isDL = "1".equals(request.getParameter("_dl"));
		ViewBean vb = null;
		if (id != null && id.length() > 0) {
			try {
				vb = this.loadViewBean(id);
			} catch (SQLException e) {
				throw new ServletException(e);
			}
		}
		if (vb == null) {
			response.sendError(404);
			return;
		}
		sortPaperAndPatent(vb);
		byte[] obs = this.buildPdf(vb);
		response.setContentType("application/pdf");
		if (isDL) {
			String fn = URLEncoder.encode(vb.getName() + ".pdf", "UTF-8");
			response.addHeader("Content-Disposition", "attachment;filename=" + fn + ";filename*=utf-8\''" + fn);
		}
		response.setContentLength(obs.length);
		OutputStream out = response.getOutputStream();
		try {
			out.write(obs);
			out.flush();
		} finally {
			out.close();
		}

	}

	@Override
	public void init() throws ServletException {
		this.dataSource = JfwAppContext.getDataSource();

	}

	private byte[] buildPdf(ViewBean vb) throws IOException {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		PdfFont cnFont = null;
		PdfFont enFont = null;

		cnFont = PdfFontFactory.createFont("STSongStd-Light", "UniGB-UCS2-H", false);
		enFont = PdfFontFactory.createFont();

		Document doc = this.createDocument(os, cnFont, enFont);
		try {
			addName(doc, vb.getName(), vb.getTitle(), cnFont, enFont);
			addOrg(doc, vb.getOrgName(), vb.getDep(), vb.getOffice(), cnFont, enFont);
			addLinkData(doc, vb.getPhone(), vb.getEmail(), vb.getCity(), cnFont, enFont);
			addDescp(doc, vb.getDescp(), cnFont, enFont);

			addReseachArea(doc, vb.getReseachAreas(), cnFont, enFont);

			addProject(doc, vb.getProjects(), cnFont, enFont);

			addHonor(doc, vb.getHonors(), cnFont, enFont);

			addJob(doc, vb.getJobs(), cnFont, enFont);

			addEdu(doc, vb.getEds(), cnFont, enFont);

			addPatent(doc, vb.getPatents(), cnFont, enFont);
			addPaper(doc, vb.getPapers(), cnFont, enFont);
			doc.flush();

		} finally {
			doc.close();
		}

		os.flush();
		return os.toByteArray();
	}

	private void addName(Document doc, String name, String title, PdfFont cnFont, PdfFont enFont) {
		Paragraph pg = new Paragraph();
		addText(pg, name, 20f, cnFont, enFont);
		if (title != null && title.length() > 0) {
			addText(pg, " " + title, 12f, cnFont, enFont);
		}
		pg.setFontColor(RGB3);
		pg.setMargin(0f);
		pg.setMultipliedLeading(1.5f);
		doc.add(pg);
	}

	private void addOrg(Document doc, String org, String dep, String office, PdfFont cnFont, PdfFont enFont) {
		StringBuilder sb = new StringBuilder();
		sb.append(org);
		if (dep != null && dep.length() > 0) {
			sb.append(" , ").append(dep);
		}
		if (office != null && office.length() > 0) {
			sb.append(" · ").append(office);
		}
		Paragraph pg = new Paragraph();
		pg.setFontSize(TITLE_FONT_SIZE);
		pg.setFontColor(RGB3);
		pg.setMargin(0f);
		pg.setMultipliedLeading(LINE_INTERVAL_L);
		addText(pg, sb.toString(), null, cnFont, enFont);
		doc.add(pg);
	}

	private void addLinkData(Document doc, String phone, String email, String city, PdfFont cnFont, PdfFont enFont) {
		StringBuilder sb = new StringBuilder();
		if (phone != null && phone.trim().length() > 0) {
			sb.append(phone.trim());
		}
		if (email != null && email.trim().length() > 0) {
			if (sb.length() > 0) {
				sb.append(" | ");
			}
			sb.append(email.trim());
		}
		if (city != null && city.trim().length() > 0) {
			if (sb.length() > 0) {
				sb.append(" | ");
			}
			sb.append(city.trim());
		}
		if (sb.length() > 0) {
			Paragraph pg = new Paragraph();
			pg.setFontSize(TEXT_FONT_SIZE);
			pg.setFontColor(RGB9);
			pg.setMargin(0f);
			pg.setMultipliedLeading(LINE_INTERVAL_L);
			addText(pg, sb.toString(), null, cnFont, enFont);
			doc.add(pg);
		}
	}

	private void addTitle(Document doc, String title, PdfFont cnFont, PdfFont enFont) {
		Div div = new Div();
		div.setWidthPercent(100f);
		div.setBorderTop(new SolidBorder(RGBC, 1f));
		div.setHeight(1f);
		div.setMargins(10f, 0f, 10f, 0f);
		doc.add(div);
		Paragraph pg = createParagraph(TITLE_FONT_SIZE, RGB9);
		addText(pg, title, null, cnFont, enFont);
		doc.add(pg);
	}

	private void addDescp(Document doc, String descp, PdfFont cnFont, PdfFont enFont) {
		if (descp != null && descp.length() > 0) {
			addTitle(doc, "个人简介", cnFont, enFont);
			Paragraph pg = createParagraph(TEXT_FONT_SIZE, RGB3);
			addText(pg, descp, null, cnFont, enFont);
			doc.add(pg);
		}
	}

	private void addReseachArea(Document doc, java.util.List<String> ras, PdfFont cnFont, PdfFont enFont) {
		if (ras != null && ras.size() > 0) {
			addTitle(doc, "研究方向", cnFont, enFont);
			List list = new List().setSymbolIndent(10).setListSymbol("\u2022").setFont(enFont);
			for (String s : ras) {
				Paragraph pg = createParagraph(TEXT_FONT_SIZE, RGB3);
				addText(pg, s, null, cnFont, enFont);
				ListItem li = new ListItem();
				li.add(pg);
				list.add(li);
			}
			doc.add(list);
		}
	}

	private void addProject(Document doc, java.util.List<Project> projects, PdfFont cnFont, PdfFont enFont) {
		if (projects != null && projects.size() > 0) {
			addTitle(doc, "项目经历", cnFont, enFont);
			List list = new List().setSymbolIndent(10).setListSymbol("\u2022").setFont(enFont);
			for (Project p : projects) {
				ListItem li = new ListItem();
				Paragraph pg = createParagraph(TEXT_FONT_SIZE, RGB3);
				addText(pg, p.getName(), null, cnFont, enFont);
				li.add(pg);
				String bm = p.getStartMonth();
				String em = p.getStopMonth();
				if (bm != null) {
					StringBuilder sb = new StringBuilder();
					sb.append(bm.substring(0, 4)).append("年").append(bm.substring(4)).append("月").append(" – ");
					if (em != null) {
						sb.append(em.substring(0, 4)).append("年").append(em.substring(4)).append("月");
					} else {
						sb.append("至今");
					}
					pg = createParagraph(TEXT_FONT_SIZE, RGB9);
					addText(pg, sb.toString(), null, cnFont, enFont);
					li.add(pg);
				}
				String descp = p.getDescp();
				if (descp != null && descp.length() > 0) {
					pg = createParagraph(TEXT_FONT_SIZE, RGB9);
					addText(pg, descp, null, cnFont, enFont);
					li.add(pg);
				}
				list.add(li);
			}
			doc.add(list);
		}
	}

	private void addHonor(Document doc, java.util.List<Honor> honors, PdfFont cnFont, PdfFont enFont) {
		if (honors != null && honors.size() > 0) {
			addTitle(doc, "荣誉奖项", cnFont, enFont);
			List list = new List().setSymbolIndent(10).setListSymbol("\u2022").setFont(enFont);
			for (Honor h : honors) {
				ListItem li = new ListItem();
				Paragraph pg = createParagraph(TEXT_FONT_SIZE, RGB3);
				addText(pg, h.getName(), null, cnFont, enFont);
				li.add(pg);
				String year = h.getYear();
				if (year != null) {
					pg = createParagraph(TEXT_FONT_SIZE, RGB9);
					addText(pg, ("获得于" + year + "年 "), null, cnFont, enFont);
					li.add(pg);
				}
				String descp = h.getDescp();
				if (descp != null && descp.length() > 0) {
					pg = createParagraph(TEXT_FONT_SIZE, RGB9);
					addText(pg, descp, null, cnFont, enFont);
					li.add(pg);
				}
				list.add(li);
			}
			doc.add(list);
		}
	}

	private void addJob(Document doc, java.util.List<PartTimeJob> jobs, PdfFont cnFont, PdfFont enFont) {
		if (jobs != null && jobs.size() > 0) {
			addTitle(doc, "工作经历", cnFont, enFont);
			List list = new List().setSymbolIndent(10).setListSymbol("\u2022").setFont(enFont);
			for (PartTimeJob p : jobs) {
				StringBuilder sb = new StringBuilder();
				sb.append(p.getCompany());
				String dep = p.getDepartment();
				if (dep != null && dep.length() > 0) {
					sb.append(" , ").append(dep);
				}
				String office = p.getTitle();
				if (office != null && office.length() > 0) {
					sb.append(" · ").append(office);
				}

				ListItem li = new ListItem();
				Paragraph pg = createParagraph(TEXT_FONT_SIZE, RGB3);
				addText(pg, sb.toString(), null, cnFont, enFont);
				li.add(pg);

				String bm = p.getStartMonth();
				String em = p.getStopMonth();
				if (bm != null) {
					pg = createParagraph(TEXT_FONT_SIZE, RGB9);
					sb.delete(0, sb.length());
					sb.append(bm.substring(0, 4)).append("年").append(bm.substring(4)).append("月 – ");
					if (em != null) {
						sb.append(em.substring(0, 4)).append("年").append(em.substring(4)).append("月");
					} else {
						sb.append("至今");
					}
					addText(pg, sb.toString(), null, cnFont, enFont);
					li.add(pg);
				}
				list.add(li);
			}
			doc.add(list);
		}
	}

	private void addEdu(Document doc, java.util.List<ProfessorEduBg> edus, PdfFont cnFont, PdfFont enFont) {
		if (edus != null && edus.size() > 0) {
			addTitle(doc, "教育背景", cnFont, enFont);
			List list = new List().setSymbolIndent(10).setListSymbol("\u2022").setFont(enFont);
			for (ProfessorEduBg h : edus) {
				ListItem li = new ListItem();
				StringBuilder sb = new StringBuilder();
				sb.append(h.getSchool());
				String college = h.getCollege();
				if (college != null && college.length() > 0) {
					sb.append(" , ").append(college);
				}
				String major = h.getMajor();
				if (major != null && major.length() > 0) {
					sb.append(" , ").append(major);
				}
				String office = h.getDegree();
				if (office != null && office.length() > 0) {
					sb.append(" · ").append(office);
				}
				Paragraph pg = createParagraph(TEXT_FONT_SIZE, RGB3);
				addText(pg, sb.toString(), null, cnFont, enFont);
				li.add(pg);

				String year = h.getYear();
				if (year != null && year.length() == 0) {
					year = null;
				}
				pg = createParagraph(TEXT_FONT_SIZE, RGB9);
				addText(pg, (year == null ? "至今" : ("获得于" + year + "年 ")), null, cnFont, enFont);
				li.add(pg);
				list.add(li);
			}
			doc.add(list);
		}
	}

	private void addPatent(Document doc, java.util.List<Ppatent> pats, PdfFont cnFont, PdfFont enFont) {
		if (pats != null && pats.size() > 0) {
			addTitle(doc, "专利", cnFont, enFont);
			List list = new List().setSymbolIndent(10).setListSymbol("\u2022").setFont(enFont);

			int len = Math.min(20, pats.size());

			for (int i = 0; i < len; ++i) {
				Ppatent p = pats.get(i);
				ListItem li = new ListItem();
				StringBuilder sb = new StringBuilder();
				sb.append(p.getAuthors().replaceAll(",", ";"));
				if (sb.charAt(sb.length() - 1) == ';') {
					sb.setCharAt(sb.length() - 1, '.');
				} else {
					sb.append('.');
				}
				sb.append(" ").append(p.getName()).append(" [P]. 中国专利: ").append(p.getCode()).append(", ").append(p.getPubDay().substring(0, 4)).append("-")
						.append(p.getPubDay().substring(4, 6)).append("-").append(p.getPubDay().substring(6)).append(".");
				Paragraph pg = createParagraph(TEXT_FONT_SIZE, RGB3);
				addText(pg, sb.toString(), null, cnFont, enFont);
				li.add(pg);
				list.add(li);
			}
			doc.add(list);
		}
	}

	private void addPaper(Document doc, java.util.List<Ppaper> paps, PdfFont cnFont, PdfFont enFont) {
		if (paps != null && paps.size() > 0) {
			addTitle(doc, "论文", cnFont, enFont);
			List list = new List().setSymbolIndent(10).setListSymbol("\u2022").setFont(enFont);
			int len = Math.min(20, paps.size());

			for (int i = 0; i < len; ++i) {
				Ppaper p = paps.get(i);
				ListItem li = new ListItem();
				StringBuilder sb = new StringBuilder();
				sb.append(p.getName()).append(" [J]. ").append(p.getAuthors());
				if (sb.charAt(sb.length() - 1) == ',') {
					sb.delete(sb.length() - 1, sb.length());
				}
				String cn = p.getCn4periodical();
				String en = p.getEn4periodical();
				if (cn != null && cn.trim().length() > 0) {
					sb.append(". ").append(cn.trim());
				} else if (en != null && en.trim().length() > 0) {
					sb.append(". ").append(en.trim());
				}
				String pd = p.getPubDay();
				if (pd != null) {
					pd = pd.trim();
					if (pd.length() == 8) {
						sb.append(".").append(pd.substring(0, 4)).append("(").append(pd.substring(5, 7)).append(")");
					}
				}
				Paragraph pg = createParagraph(TEXT_FONT_SIZE, RGB3);
				addText(pg, sb.toString(), null, cnFont, enFont);
				li.add(pg);
				list.add(li);
			}
			doc.add(list);
		}

	}

	private Document createDocument(OutputStream out, PdfFont cnFont, PdfFont enFont) {
		PdfWriter writer = new PdfWriter(out);
		PdfDocument pdf = new PdfDocument(writer);
		pdf.addEventHandler(PdfDocumentEvent.END_PAGE, new MyEventHandler(cnFont, enFont));
		Document doc = new Document(pdf, PageSize.A4);
		doc.setMargins(42.5f, 56.5f, 42.5f, 56.5f);
		return doc;
	}

	private Paragraph createParagraph(float fontSize, Color fontColor) {
		Paragraph pg = new Paragraph();
		pg.setFontSize(fontSize);
		pg.setFontColor(fontColor);
		pg.setMargin(0f);
		pg.setMultipliedLeading(LINE_INTERVAL_M);
		return pg;
	}

	private void addText(Paragraph p, String str, Float fontSize, PdfFont cnFont, PdfFont enFont) {
		int b = 0;
		char[] cs = str.toCharArray();
		PdfFont cur = cnFont;
		PdfFont oth = enFont;
		if (cs.length > 0) {
			for (int i = 0; i < cs.length; ++i) {
				char c = cs[i];
				if (c < 256) {
					if (cur == enFont) {
						continue;
					} else {
						if (i > b) {
							Text t = new Text(new String(cs, b, i - b)).setFont(cur);
							if (fontSize != null) {
								t.setFontSize(fontSize.floatValue());
							}
							p.add(t);
						}
						cur = enFont;
						oth = cnFont;
						b = i;
					}
				} else {
					if (!cur.containsGlyph(c)) {
						if (!oth.containsGlyph(c)) {
							cs[i] = ' ';
						} else {
							if (i > b) {
								Text t = new Text(new String(cs, b, i - b)).setFont(cur);
								if (fontSize != null) {
									t.setFontSize(fontSize.floatValue());
								}
								p.add(t);
							}
							b = i;
							if (cur == cnFont) {
								cur = enFont;
								oth = cnFont;
							} else {
								cur = cnFont;
								oth = enFont;
							}
						}
					}
				}
			}
			Text tt = new Text(new String(cs, b, cs.length - b)).setFont(cur);
			if (fontSize != null) {
				tt.setFontSize(fontSize.floatValue());
			}
			p.add(tt);
		}

	}

	private Connection getConnection() throws SQLException {
		return this.dataSource.getConnection();
		// return
		// DriverManager.getConnection("jdbc:postgresql://www.ekexiu.com:5432/ekexiu",
		// "postgres", "postgres");
	}

	private ViewBean loadViewBean(final String id) throws SQLException {
		Connection con = this.getConnection();
		try {
			final ViewBean vb = JdbcUtil.query(con, "SELECT NAME,ORG_ID,TITLE,OFFICE,DEPARTMENT,DESCP,PHONE,EMAIL,ADDRESS FROM PROFESSOR WHERE ID=?",
					new ResultSetExtractor<ViewBean>() {

						@Override
						public ViewBean extractData(ResultSet rs) throws SQLException {
							if (!rs.next())
								return null;
							ViewBean v = new ViewBean();
							v.setName(rs.getString(1));
							v.setOrgId(rs.getString(2));
							v.setTitle(rs.getString(3));
							v.setOffice(rs.getString(4));
							v.setDep(rs.getString(5));
							v.setDescp(rs.getString(6));
							v.setPhone(rs.getString(7));
							v.setEmail(rs.getString(8));
							v.setCity(rs.getString(9));
							return v;
						}
					}, new PreparedStatementConfig() {

						@Override
						public void config(PreparedStatement ps) throws SQLException {
							ps.setString(1, id);

						}
					});
			if (vb == null) {
				return null;
			}
			JdbcUtil.query(con, "SELECT NAME FROM organization WHERE ID=?", new ResultSetExtractor<ViewBean>() {
				@Override
				public ViewBean extractData(ResultSet rs) throws SQLException {
					if (!rs.next())
						return null;
					vb.setOrgName(rs.getString(1));
					return vb;
				}
			}, new PreparedStatementConfig() {

				@Override
				public void config(PreparedStatement ps) throws SQLException {
					ps.setString(1, vb.getOrgId());

				}
			});
			vb.setReseachAreas(JdbcUtil.queryList(con, "SELECT CAPTION,SORT_NUM FROM research_area WHERE PROFESSOR_ID=? ORDER BY SORT_NUM DESC",
					new ResultSetExtractor<String>() {
						@Override
						public String extractData(ResultSet rs) throws SQLException {
							return rs.getString(1);
						}

					}, new PreparedStatementConfig() {
						@Override
						public void config(PreparedStatement ps) throws SQLException {
							ps.setString(1, id);
							;
						}
					}));
			vb.setProjects(
					JdbcUtil.queryList(con, "SELECT start_month,stop_month,name,descp FROM project WHERE PROFESSOR_ID=?", new ResultSetExtractor<Project>() {
						@Override
						public Project extractData(ResultSet rs) throws SQLException {
							Project p = new Project();
							p.setStartMonth(rs.getString(1));
							p.setStopMonth(rs.getString(2));
							p.setName(rs.getString(3));
							p.setDescp(rs.getString(4));
							return p;
						}

					}, new PreparedStatementConfig() {
						@Override
						public void config(PreparedStatement ps) throws SQLException {
							ps.setString(1, id);
							;
						}
					}));
			vb.setHonors(JdbcUtil.queryList(con, "SELECT year,name,descp FROM honor WHERE PROFESSOR_ID=?", new ResultSetExtractor<Honor>() {
				@Override
				public Honor extractData(ResultSet rs) throws SQLException {
					Honor p = new Honor();
					p.setYear(rs.getString(1));
					p.setName(rs.getString(2));
					p.setDescp(rs.getString(3));
					return p;
				}

			}, new PreparedStatementConfig() {
				@Override
				public void config(PreparedStatement ps) throws SQLException {
					ps.setString(1, id);
					;
				}
			}));
			vb.setJobs(JdbcUtil.queryList(con, "SELECT start_month,stop_month,company,title,department FROM part_time_job WHERE PROFESSOR_ID=?",
					new ResultSetExtractor<PartTimeJob>() {
						@Override
						public PartTimeJob extractData(ResultSet rs) throws SQLException {
							PartTimeJob p = new PartTimeJob();
							p.setStartMonth(rs.getString(1));
							p.setStopMonth(rs.getString(2));
							p.setCompany(rs.getString(3));
							p.setTitle(rs.getString(4));
							p.setDepartment(rs.getString(5));
							return p;
						}

					}, new PreparedStatementConfig() {
						@Override
						public void config(PreparedStatement ps) throws SQLException {
							ps.setString(1, id);
							;
						}
					}));

			vb.setEds(JdbcUtil.queryList(con, "SELECT 	year,school,college,major,degree FROM professor_edu_bg WHERE PROFESSOR_ID=?",
					new ResultSetExtractor<ProfessorEduBg>() {
						@Override
						public ProfessorEduBg extractData(ResultSet rs) throws SQLException {
							ProfessorEduBg p = new ProfessorEduBg();
							p.setYear(rs.getString(1));
							p.setSchool(rs.getString(2));
							p.setCollege(rs.getString(3));
							p.setMajor(rs.getString(4));
							p.setDegree(rs.getString(5));
							return p;
						}

					}, new PreparedStatementConfig() {
						@Override
						public void config(PreparedStatement ps) throws SQLException {
							ps.setString(1, id);
							;
						}
					}));
			vb.setPapers(JdbcUtil.queryList(con,
					"SELECT name,cn4periodical,en4periodical,pub_day,authors FROM ppaper WHERE ID IN (SELECT PAPER_ID FROM PAPER_AUTHOR WHERE PROFESSOR_ID=?)",
					new ResultSetExtractor<Ppaper>() {
						@Override
						public Ppaper extractData(ResultSet rs) throws SQLException {
							Ppaper p = new Ppaper();
							p.setName(rs.getString(1));
							p.setCn4periodical(rs.getString(2));
							p.setEn4periodical(rs.getString(3));
							p.setPubDay(rs.getString(4));
							p.setAuthors(rs.getString(5));
							return p;
						}

					}, new PreparedStatementConfig() {
						@Override
						public void config(PreparedStatement ps) throws SQLException {
							ps.setString(1, id);
							;
						}
					}));
			vb.setPatents(JdbcUtil.queryList(con,
					"SELECT name,code,pub_day,authors FROM ppatent WHERE ID IN (SELECT PATENT_ID FROM PATENT_AUTHOR WHERE PROFESSOR_ID=?)",
					new ResultSetExtractor<Ppatent>() {
						@Override
						public Ppatent extractData(ResultSet rs) throws SQLException {
							Ppatent p = new Ppatent();
							p.setName(rs.getString(1));
							p.setCode(rs.getString(2));
							p.setPubDay(rs.getString(3));
							p.setAuthors(rs.getString(4));
							return p;
						}

					}, new PreparedStatementConfig() {
						@Override
						public void config(PreparedStatement ps) throws SQLException {
							ps.setString(1, id);
							;
						}
					}));

			return vb;
		} finally {
			JdbcUtil.close(con);
		}
	}

	private static class ViewBean {
		private String name;
		private String orgId;
		private String orgName;
		private String title;
		private String dep;
		private String descp;
		private String office;
		private String phone;
		private String email;
		private String city;
		private java.util.List<String> reseachAreas;
		private java.util.List<Project> projects;
		private java.util.List<Honor> honors;
		private java.util.List<PartTimeJob> jobs;
		private java.util.List<ProfessorEduBg> eds;
		private java.util.List<Ppaper> papers;
		private java.util.List<Ppatent> patents;

		public String getPhone() {
			return phone;
		}

		public void setPhone(String phone) {
			this.phone = phone;
		}

		public String getEmail() {
			return email;
		}

		public void setEmail(String email) {
			this.email = email;
		}

		public String getCity() {
			return city;
		}

		public void setCity(String city) {
			this.city = city;
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public String getDescp() {
			return descp;
		}

		public void setDescp(String descp) {
			this.descp = descp;
		}

		public String getOrgId() {
			return orgId;
		}

		public void setOrgId(String orgId) {
			this.orgId = orgId;
		}

		public String getOrgName() {
			return orgName;
		}

		public void setOrgName(String orgName) {
			this.orgName = orgName;
		}

		public String getTitle() {
			return title;
		}

		public void setTitle(String title) {
			this.title = title;
		}

		public String getDep() {
			return dep;
		}

		public void setDep(String dep) {
			this.dep = dep;
		}

		public String getOffice() {
			return office;
		}

		public void setOffice(String office) {
			this.office = office;
		}

		public java.util.List<String> getReseachAreas() {
			return reseachAreas;
		}

		public void setReseachAreas(java.util.List<String> reseachAreas) {
			this.reseachAreas = reseachAreas;
		}

		public java.util.List<Project> getProjects() {
			return projects;
		}

		public void setProjects(java.util.List<Project> projects) {
			this.projects = projects;
		}

		public java.util.List<Honor> getHonors() {
			return honors;
		}

		public void setHonors(java.util.List<Honor> honors) {
			this.honors = honors;
		}

		public java.util.List<PartTimeJob> getJobs() {
			return jobs;
		}

		public void setJobs(java.util.List<PartTimeJob> jobs) {
			this.jobs = jobs;
		}

		public java.util.List<ProfessorEduBg> getEds() {
			return eds;
		}

		public void setEds(java.util.List<ProfessorEduBg> eds) {
			this.eds = eds;
		}

		public java.util.List<Ppaper> getPapers() {
			return papers;
		}

		public void setPapers(java.util.List<Ppaper> papers) {
			this.papers = papers;
		}

		public java.util.List<Ppatent> getPatents() {
			return patents;
		}

		public void setPatents(java.util.List<Ppatent> patents) {
			this.patents = patents;
		}

	}

	protected class MyEventHandler implements IEventHandler {
		private PdfFont cnFont;
		private PdfFont enFont;

		public MyEventHandler(PdfFont cnFont, PdfFont enFont) {
			this.cnFont = cnFont;
			this.enFont = enFont;
		}

		public void handleEvent(Event event) {
			PdfDocumentEvent docEvent = (PdfDocumentEvent) event;
			PdfDocument pdfDoc = docEvent.getDocument();
			PdfPage page = docEvent.getPage();
			Rectangle pageSize = page.getPageSize();
			PdfCanvas pdfCanvas = new PdfCanvas(page.newContentStreamBefore(), page.getResources(), pdfDoc);
			pdfCanvas.beginText().setFontAndSize(cnFont, 10).setColor(new DeviceRgb(0x99, 0x99, 0x99), true)
					.setTextMatrix(1, 0, 0.51256f, 1, pageSize.getWidth() / 2 - 72, pageSize.getBottom() + 20).showText("简历来自科袖网 - www.ekexiu.com").endText();
			pdfCanvas.release();
		}
	}
}
