package com.ekexiu.portal.views.service;

import com.ekexiu.portal.views.dao.DayDao;
import com.ekexiu.portal.views.handler.LogHandler;
import com.ekexiu.portal.views.pojo.DayData;
import com.ekexiu.portal.views.pojo.PpatentSum;
import com.ekexiu.portal.views.pojo.ResourceSum;
import com.ekexiu.portal.views.pojo.WareSum;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.ParameterMap;
import org.jfw.util.PageQueryResult;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.jdbc.PreparedStatementConfig;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

@Path("/log")
public class Main {
    public static final String MAX_DAY = "99999999";
    public static final String MIN_DAY = "00000000";


    private List<String> tns;
    private List<LogHandler> handlers;

    @Autowrie
    private DayDao dayDao;

    public List<String> getTns() {
        return tns;
    }

    public void setTns(List<String> tns) {
        this.tns = tns;
    }

    public List<LogHandler> getHandlers() {
        return handlers;
    }

    public void setHandlers(List<LogHandler> handlers) {
        this.handlers = handlers;
    }

    public DayDao getDayDao() {
        return dayDao;
    }

    public void setDayDao(DayDao dayDao) {
        this.dayDao = dayDao;
    }

    @Get
    @Path()
    public void log(@ParameterMap Map<String, String[]> pm) {
        for (LogHandler lh : this.handlers) {
            lh.add(pm);
        }
    }

    @Get
    @Path("/qm")
    public List<DayData> query(@JdbcConn Connection con, String tn,String platform, @Nullable String[] src, @Nullable String[] id, @DefaultValue("com.ekexiu.portal.views.service.Main.MIN_DAY") String bt,
                               @DefaultValue("com.ekexiu.portal.views.service.Main.MAX_DAY") String et, @Nullable String sort) throws SQLException, JfwBaseException {
        if (!this.tns.contains(tn)) {
            throw new JfwBaseException(-101, "unavailable tn");
        }
        return dayDao.query(con, tn,platform, src, id, bt, et, sort);
    }

    @Get
    @Path("/qo")
    public List<DayData> query(@JdbcConn Connection con, String tn,String platform, String id, @DefaultValue("com.ekexiu.portal.views.service.Main.MIN_DAY") String bt,
                               @DefaultValue("com.ekexiu.portal.views.service.Main.MAX_DAY") String et) throws SQLException, JfwBaseException {
        if (!this.tns.contains(tn)) {
            throw new JfwBaseException(-101, "unavailable tn");
        }
        return dayDao.query(con, tn,platform, id, bt, et);
    }

    @Get
    @Path("/qo/sum")
    public int query(@JdbcConn Connection con, String tn, final String platform, final String id) throws SQLException,JfwBaseException {
        if (!this.tns.contains(tn)) {
            throw new JfwBaseException(-101, "unavailable tn");
        }
        return JdbcUtil.queryInt(con, "SELECT SUM FROM " + tn + "_DAY_SUM WHERE PLATFORM = ? AND ID = ?", new PreparedStatementConfig() {
            @Override
            public void config(PreparedStatement preparedStatement) throws SQLException {
                preparedStatement.setString(1,platform);
                preparedStatement.setString(2, id);
            }
        }, 0);
    }

    @Get
    @Path("/qs")
    public List<DayData> query(@JdbcConn Connection con, String tn,String platform, @Nullable String[] src, @DefaultValue("com.ekexiu.portal.views.service.Main.MIN_DAY") String bt,
                               @DefaultValue("com.ekexiu.portal.views.service.Main.MAX_DAY") String et, @DefaultValue("20") int rows) throws SQLException, JfwBaseException {
        if (!this.tns.contains(tn)) {
            throw new JfwBaseException(-101, "unavailable tn");
        }
        return dayDao.query(con, tn,platform, bt, et, src, rows);
    }

    @Get
    @Path("/total")
    public List<Map<String, Object>> total(@JdbcConn Connection con, String tn, final String platform, @DefaultValue("com.ekexiu.portal.views.service.Main.MIN_DAY") final String bt) throws SQLException, JfwBaseException {
        if (!this.tns.contains(tn)) {
            throw new JfwBaseException(-101, "unavailable tn");
        }
        return JdbcUtil.queryMaps(con, "SELECT day,sum(pv)as pv,source FROM "+tn+"_day WHERE PLATFORM = ? AND day>=? GROUP BY day,source ORDER BY day DESC ", new PreparedStatementConfig() {
            @Override
            public void config(PreparedStatement preparedStatement) throws SQLException {
                preparedStatement.setString(1,platform);
                preparedStatement.setString(2, bt);
            }
        });
    }


    @Get
    @Path("/ware/pq")
    public PageQueryResult<WareSum> queryWare(@JdbcConn Connection con, String platform, @Nullable String name,@Nullable String author, @Nullable String bt, @Nullable String et, String orderBy, int pageSize, int pageNo) throws SQLException {
        if (orderBy.equals("1")) {
            return dayDao.queryWareTime(con, platform, name == null ? null : "%" + name + "%", author == null ? null : "%" + author + "%", bt, et, pageSize, pageNo);
        } else if (orderBy.equals("2")) {
            return dayDao.queryWareViews(con, platform, name == null ? null : "%" + name + "%", author == null ? null : "%" + author + "%",  bt, et, pageSize, pageNo);
        }
        return null;
    }

    @Get
    @Path("/ppatent/pq")
    public PageQueryResult<PpatentSum> queryPpatent(@JdbcConn Connection con, String platform, @Nullable String name, @Nullable String author, @Nullable String bt, @Nullable String et, String orderBy, int pageSize, int pageNo) throws SQLException {
        if (orderBy.equals("1")) {
            return dayDao.queryPpatentTime(con, platform, name == null ? null : "%" + name + "%", author == null ? null : "%" + author + "%", bt, et, pageSize, pageNo);
        } else if (orderBy.equals("2")) {
            return dayDao.queryPpatentViews(con, platform, name == null ? null : "%" + name + "%", author == null ? null : "%" + author + "%",  bt, et, pageSize, pageNo);
        }
        return null;
    }

    @Get
    @Path("/resource/pq")
    public PageQueryResult<ResourceSum> queryResource(@JdbcConn Connection con, String platform, @Nullable String name, @Nullable String author, @Nullable String bt, @Nullable String et, String orderBy, int pageSize, int pageNo) throws SQLException {
        if (orderBy.equals("1")) {
            return dayDao.queryResourceTime(con, platform, name == null ? null : "%" + name + "%", author == null ? null : "%" + author + "%", bt, et, pageSize, pageNo);
        } else if (orderBy.equals("2")) {
            return dayDao.queryResourceViews(con, platform, name == null ? null : "%" + name + "%", author == null ? null : "%" + author + "%",  bt, et, pageSize, pageNo);
        }
        return null;
    }








}
